from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.agent import Agent
from ...models.get_agent_details import GetAgentDetails
from ...types import UNSET, Response, Unset


def _get_kwargs(
    matricule: str,
    *,
    details: Union[Unset, None, GetAgentDetails] = UNSET,
) -> Dict[str, Any]:
    pass

    params: Dict[str, Any] = {}
    json_details: Union[Unset, None, str] = UNSET
    if not isinstance(details, Unset):
        json_details = details.value if details else None

    params["details"] = json_details

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": "/agents/{matricule}/full".format(
            matricule=matricule,
        ),
        "params": params,
    }


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Agent]:
    if response.status_code == HTTPStatus.OK:
        response_200 = Agent.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Agent]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    matricule: str,
    *,
    client: AuthenticatedClient,
    details: Union[Unset, None, GetAgentDetails] = UNSET,
) -> Response[Agent]:
    """Récupère les informations détaillées sur un agent

    Args:
        matricule (str):
        details (Union[Unset, None, GetAgentDetails]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Agent]
    """

    kwargs = _get_kwargs(
        matricule=matricule,
        details=details,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    matricule: str,
    *,
    client: AuthenticatedClient,
    details: Union[Unset, None, GetAgentDetails] = UNSET,
) -> Optional[Agent]:
    """Récupère les informations détaillées sur un agent

    Args:
        matricule (str):
        details (Union[Unset, None, GetAgentDetails]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Agent
    """

    return sync_detailed(
        matricule=matricule,
        client=client,
        details=details,
    ).parsed


async def asyncio_detailed(
    matricule: str,
    *,
    client: AuthenticatedClient,
    details: Union[Unset, None, GetAgentDetails] = UNSET,
) -> Response[Agent]:
    """Récupère les informations détaillées sur un agent

    Args:
        matricule (str):
        details (Union[Unset, None, GetAgentDetails]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Agent]
    """

    kwargs = _get_kwargs(
        matricule=matricule,
        details=details,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    matricule: str,
    *,
    client: AuthenticatedClient,
    details: Union[Unset, None, GetAgentDetails] = UNSET,
) -> Optional[Agent]:
    """Récupère les informations détaillées sur un agent

    Args:
        matricule (str):
        details (Union[Unset, None, GetAgentDetails]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Agent
    """

    return (
        await asyncio_detailed(
            matricule=matricule,
            client=client,
            details=details,
        )
    ).parsed
