from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.security_input_set import SecurityInputSet
from ...models.security_outpu_secret import SecurityOutpuSecret
from ...types import Response


def _get_kwargs(
    *,
    json_body: SecurityInputSet,
) -> Dict[str, Any]:
    pass

    json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": "/security/set",
        "json": json_json_body,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[SecurityOutpuSecret]:
    if response.status_code == HTTPStatus.CREATED:
        response_201 = SecurityOutpuSecret.from_dict(response.json())

        return response_201
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[SecurityOutpuSecret]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    json_body: SecurityInputSet,
) -> Response[SecurityOutpuSecret]:
    """Enregistre des informations de connexion (si le compte existe déjà, les jetons existants sont
    systématiquement invalidés)

    Args:
        json_body (SecurityInputSet): Paramètres pour créer des informations de connexion

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[SecurityOutpuSecret]
    """

    kwargs = _get_kwargs(
        json_body=json_body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    json_body: SecurityInputSet,
) -> Optional[SecurityOutpuSecret]:
    """Enregistre des informations de connexion (si le compte existe déjà, les jetons existants sont
    systématiquement invalidés)

    Args:
        json_body (SecurityInputSet): Paramètres pour créer des informations de connexion

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        SecurityOutpuSecret
    """

    return sync_detailed(
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    json_body: SecurityInputSet,
) -> Response[SecurityOutpuSecret]:
    """Enregistre des informations de connexion (si le compte existe déjà, les jetons existants sont
    systématiquement invalidés)

    Args:
        json_body (SecurityInputSet): Paramètres pour créer des informations de connexion

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[SecurityOutpuSecret]
    """

    kwargs = _get_kwargs(
        json_body=json_body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    json_body: SecurityInputSet,
) -> Optional[SecurityOutpuSecret]:
    """Enregistre des informations de connexion (si le compte existe déjà, les jetons existants sont
    systématiquement invalidés)

    Args:
        json_body (SecurityInputSet): Paramètres pour créer des informations de connexion

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        SecurityOutpuSecret
    """

    return (
        await asyncio_detailed(
            client=client,
            json_body=json_body,
        )
    ).parsed
