from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.discipline import Discipline
from ...types import Response


def _get_kwargs(
    speciality: str,
    code: str,
) -> Dict[str, Any]:
    pass

    return {
        "method": "get",
        "url": "/disciplines/{speciality}/{code}".format(
            speciality=speciality,
            code=code,
        ),
    }


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Discipline]:
    if response.status_code == HTTPStatus.OK:
        response_200 = Discipline.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Discipline]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    speciality: str,
    code: str,
    *,
    client: AuthenticatedClient,
) -> Response[Discipline]:
    """Récupère le détail sur la discipline d'une spécialité de niveau 1

    Args:
        speciality (str):
        code (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Discipline]
    """

    kwargs = _get_kwargs(
        speciality=speciality,
        code=code,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    speciality: str,
    code: str,
    *,
    client: AuthenticatedClient,
) -> Optional[Discipline]:
    """Récupère le détail sur la discipline d'une spécialité de niveau 1

    Args:
        speciality (str):
        code (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Discipline
    """

    return sync_detailed(
        speciality=speciality,
        code=code,
        client=client,
    ).parsed


async def asyncio_detailed(
    speciality: str,
    code: str,
    *,
    client: AuthenticatedClient,
) -> Response[Discipline]:
    """Récupère le détail sur la discipline d'une spécialité de niveau 1

    Args:
        speciality (str):
        code (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Discipline]
    """

    kwargs = _get_kwargs(
        speciality=speciality,
        code=code,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    speciality: str,
    code: str,
    *,
    client: AuthenticatedClient,
) -> Optional[Discipline]:
    """Récupère le détail sur la discipline d'une spécialité de niveau 1

    Args:
        speciality (str):
        code (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Discipline
    """

    return (
        await asyncio_detailed(
            speciality=speciality,
            code=code,
            client=client,
        )
    ).parsed
