from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.discipline import Discipline
from ...types import UNSET, Response, Unset


def _get_kwargs(
    speciality: str,
    *,
    all_: Union[Unset, None, bool] = UNSET,
) -> Dict[str, Any]:
    pass

    params: Dict[str, Any] = {}
    params["all"] = all_

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": "/disciplines/{speciality}".format(
            speciality=speciality,
        ),
        "params": params,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[List["Discipline"]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = Discipline.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[List["Discipline"]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    speciality: str,
    *,
    client: AuthenticatedClient,
    all_: Union[Unset, None, bool] = UNSET,
) -> Response[List["Discipline"]]:
    """Récupère la liste des disciplines d'une spécialité de niveau 1

    Args:
        speciality (str):
        all_ (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[List['Discipline']]
    """

    kwargs = _get_kwargs(
        speciality=speciality,
        all_=all_,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    speciality: str,
    *,
    client: AuthenticatedClient,
    all_: Union[Unset, None, bool] = UNSET,
) -> Optional[List["Discipline"]]:
    """Récupère la liste des disciplines d'une spécialité de niveau 1

    Args:
        speciality (str):
        all_ (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        List['Discipline']
    """

    return sync_detailed(
        speciality=speciality,
        client=client,
        all_=all_,
    ).parsed


async def asyncio_detailed(
    speciality: str,
    *,
    client: AuthenticatedClient,
    all_: Union[Unset, None, bool] = UNSET,
) -> Response[List["Discipline"]]:
    """Récupère la liste des disciplines d'une spécialité de niveau 1

    Args:
        speciality (str):
        all_ (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[List['Discipline']]
    """

    kwargs = _get_kwargs(
        speciality=speciality,
        all_=all_,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    speciality: str,
    *,
    client: AuthenticatedClient,
    all_: Union[Unset, None, bool] = UNSET,
) -> Optional[List["Discipline"]]:
    """Récupère la liste des disciplines d'une spécialité de niveau 1

    Args:
        speciality (str):
        all_ (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        List['Discipline']
    """

    return (
        await asyncio_detailed(
            speciality=speciality,
            client=client,
            all_=all_,
        )
    ).parsed
