from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.get_labels_type import GetLabelsType
from ...models.label import Label
from ...types import UNSET, Response, Unset


def _get_kwargs(
    type: GetLabelsType,
    *,
    all_: Union[Unset, None, bool] = UNSET,
) -> Dict[str, Any]:
    pass

    params: Dict[str, Any] = {}
    params["all"] = all_

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": "/labels/{type}".format(
            type=type,
        ),
        "params": params,
    }


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[List["Label"]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = Label.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[List["Label"]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    type: GetLabelsType,
    *,
    client: AuthenticatedClient,
    all_: Union[Unset, None, bool] = UNSET,
) -> Response[List["Label"]]:
    """Récupère la liste d'un type d'étiquettes

    Args:
        type (GetLabelsType):
        all_ (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[List['Label']]
    """

    kwargs = _get_kwargs(
        type=type,
        all_=all_,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    type: GetLabelsType,
    *,
    client: AuthenticatedClient,
    all_: Union[Unset, None, bool] = UNSET,
) -> Optional[List["Label"]]:
    """Récupère la liste d'un type d'étiquettes

    Args:
        type (GetLabelsType):
        all_ (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        List['Label']
    """

    return sync_detailed(
        type=type,
        client=client,
        all_=all_,
    ).parsed


async def asyncio_detailed(
    type: GetLabelsType,
    *,
    client: AuthenticatedClient,
    all_: Union[Unset, None, bool] = UNSET,
) -> Response[List["Label"]]:
    """Récupère la liste d'un type d'étiquettes

    Args:
        type (GetLabelsType):
        all_ (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[List['Label']]
    """

    kwargs = _get_kwargs(
        type=type,
        all_=all_,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    type: GetLabelsType,
    *,
    client: AuthenticatedClient,
    all_: Union[Unset, None, bool] = UNSET,
) -> Optional[List["Label"]]:
    """Récupère la liste d'un type d'étiquettes

    Args:
        type (GetLabelsType):
        all_ (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        List['Label']
    """

    return (
        await asyncio_detailed(
            type=type,
            client=client,
            all_=all_,
        )
    ).parsed
