from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.input_mail import InputMail
from ...models.receipt import Receipt
from ...types import Response


def _get_kwargs(
    matricule: str,
    *,
    json_body: InputMail,
) -> Dict[str, Any]:
    pass

    json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": "/personal/{matricule}/mail".format(
            matricule=matricule,
        ),
        "json": json_json_body,
    }


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Receipt]:
    if response.status_code == HTTPStatus.OK:
        response_200 = Receipt.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Receipt]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    matricule: str,
    *,
    client: AuthenticatedClient,
    json_body: InputMail,
) -> Response[Receipt]:
    """Tente d'envoyer un mail personnel à un individu

    Args:
        matricule (str):
        json_body (InputMail):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Receipt]
    """

    kwargs = _get_kwargs(
        matricule=matricule,
        json_body=json_body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    matricule: str,
    *,
    client: AuthenticatedClient,
    json_body: InputMail,
) -> Optional[Receipt]:
    """Tente d'envoyer un mail personnel à un individu

    Args:
        matricule (str):
        json_body (InputMail):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Receipt
    """

    return sync_detailed(
        matricule=matricule,
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    matricule: str,
    *,
    client: AuthenticatedClient,
    json_body: InputMail,
) -> Response[Receipt]:
    """Tente d'envoyer un mail personnel à un individu

    Args:
        matricule (str):
        json_body (InputMail):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Receipt]
    """

    kwargs = _get_kwargs(
        matricule=matricule,
        json_body=json_body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    matricule: str,
    *,
    client: AuthenticatedClient,
    json_body: InputMail,
) -> Optional[Receipt]:
    """Tente d'envoyer un mail personnel à un individu

    Args:
        matricule (str):
        json_body (InputMail):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Receipt
    """

    return (
        await asyncio_detailed(
            matricule=matricule,
            client=client,
            json_body=json_body,
        )
    ).parsed
