from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.professional_contact import ProfessionalContact
from ...types import Response


def _get_kwargs(
    matricule: str,
) -> Dict[str, Any]:
    pass

    return {
        "method": "get",
        "url": "/professional/{matricule}/contacts".format(
            matricule=matricule,
        ),
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[ProfessionalContact]:
    if response.status_code == HTTPStatus.OK:
        response_200 = ProfessionalContact.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[ProfessionalContact]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    matricule: str,
    *,
    client: AuthenticatedClient,
) -> Response[ProfessionalContact]:
    """Récupère les coordonnées professionnelles d'un individu

    Args:
        matricule (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ProfessionalContact]
    """

    kwargs = _get_kwargs(
        matricule=matricule,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    matricule: str,
    *,
    client: AuthenticatedClient,
) -> Optional[ProfessionalContact]:
    """Récupère les coordonnées professionnelles d'un individu

    Args:
        matricule (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ProfessionalContact
    """

    return sync_detailed(
        matricule=matricule,
        client=client,
    ).parsed


async def asyncio_detailed(
    matricule: str,
    *,
    client: AuthenticatedClient,
) -> Response[ProfessionalContact]:
    """Récupère les coordonnées professionnelles d'un individu

    Args:
        matricule (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ProfessionalContact]
    """

    kwargs = _get_kwargs(
        matricule=matricule,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    matricule: str,
    *,
    client: AuthenticatedClient,
) -> Optional[ProfessionalContact]:
    """Récupère les coordonnées professionnelles d'un individu

    Args:
        matricule (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ProfessionalContact
    """

    return (
        await asyncio_detailed(
            matricule=matricule,
            client=client,
        )
    ).parsed
