from http import HTTPStatus
from typing import Any, Dict, Optional, Union, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.count_modifications_types import CountModificationsTypes
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    start: Union[Unset, None, str] = UNSET,
    end: Union[Unset, None, str] = UNSET,
    types: Union[Unset, None, CountModificationsTypes] = UNSET,
) -> Dict[str, Any]:
    pass

    params: Dict[str, Any] = {}
    params["start"] = start

    params["end"] = end

    json_types: Union[Unset, None, str] = UNSET
    if not isinstance(types, Unset):
        json_types = types.value if types else None

    params["types"] = json_types

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": "/modifications/count",
        "params": params,
    }


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[int]:
    if response.status_code == HTTPStatus.OK:
        response_200 = cast(int, response.json())
        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[int]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    start: Union[Unset, None, str] = UNSET,
    end: Union[Unset, None, str] = UNSET,
    types: Union[Unset, None, CountModificationsTypes] = UNSET,
) -> Response[int]:
    """Compte le nombre de modifications répertoriées

    Args:
        start (Union[Unset, None, str]):
        end (Union[Unset, None, str]):
        types (Union[Unset, None, CountModificationsTypes]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[int]
    """

    kwargs = _get_kwargs(
        start=start,
        end=end,
        types=types,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    start: Union[Unset, None, str] = UNSET,
    end: Union[Unset, None, str] = UNSET,
    types: Union[Unset, None, CountModificationsTypes] = UNSET,
) -> Optional[int]:
    """Compte le nombre de modifications répertoriées

    Args:
        start (Union[Unset, None, str]):
        end (Union[Unset, None, str]):
        types (Union[Unset, None, CountModificationsTypes]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        int
    """

    return sync_detailed(
        client=client,
        start=start,
        end=end,
        types=types,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    start: Union[Unset, None, str] = UNSET,
    end: Union[Unset, None, str] = UNSET,
    types: Union[Unset, None, CountModificationsTypes] = UNSET,
) -> Response[int]:
    """Compte le nombre de modifications répertoriées

    Args:
        start (Union[Unset, None, str]):
        end (Union[Unset, None, str]):
        types (Union[Unset, None, CountModificationsTypes]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[int]
    """

    kwargs = _get_kwargs(
        start=start,
        end=end,
        types=types,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    start: Union[Unset, None, str] = UNSET,
    end: Union[Unset, None, str] = UNSET,
    types: Union[Unset, None, CountModificationsTypes] = UNSET,
) -> Optional[int]:
    """Compte le nombre de modifications répertoriées

    Args:
        start (Union[Unset, None, str]):
        end (Union[Unset, None, str]):
        types (Union[Unset, None, CountModificationsTypes]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        int
    """

    return (
        await asyncio_detailed(
            client=client,
            start=start,
            end=end,
            types=types,
        )
    ).parsed
