from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.connector_status import ConnectorStatus
from ...models.get_connector_state_id import GetConnectorStateId
from ...types import Response


def _get_kwargs(
    id: GetConnectorStateId,
) -> Dict[str, Any]:
    pass

    return {
        "method": "get",
        "url": "/health/connections/{id}".format(
            id=id,
        ),
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[ConnectorStatus]:
    if response.status_code == HTTPStatus.OK:
        response_200 = ConnectorStatus.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[ConnectorStatus]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    id: GetConnectorStateId,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[ConnectorStatus]:
    """Récupère un état de connexion

    Args:
        id (GetConnectorStateId):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ConnectorStatus]
    """

    kwargs = _get_kwargs(
        id=id,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    id: GetConnectorStateId,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Optional[ConnectorStatus]:
    """Récupère un état de connexion

    Args:
        id (GetConnectorStateId):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ConnectorStatus
    """

    return sync_detailed(
        id=id,
        client=client,
    ).parsed


async def asyncio_detailed(
    id: GetConnectorStateId,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[ConnectorStatus]:
    """Récupère un état de connexion

    Args:
        id (GetConnectorStateId):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ConnectorStatus]
    """

    kwargs = _get_kwargs(
        id=id,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    id: GetConnectorStateId,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Optional[ConnectorStatus]:
    """Récupère un état de connexion

    Args:
        id (GetConnectorStateId):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ConnectorStatus
    """

    return (
        await asyncio_detailed(
            id=id,
            client=client,
        )
    ).parsed
