from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.absence import Absence
from ...models.get_absences_agent_details import GetAbsencesAgentDetails
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    json_body: List[str],
    details: Union[Unset, None, GetAbsencesAgentDetails] = UNSET,
    include_managements: Union[Unset, None, bool] = UNSET,
) -> Dict[str, Any]:
    pass

    params: Dict[str, Any] = {}
    json_details: Union[Unset, None, str] = UNSET
    if not isinstance(details, Unset):
        json_details = details.value if details else None

    params["details"] = json_details

    params["includeManagements"] = include_managements

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    json_json_body = json_body

    return {
        "method": "post",
        "url": "/ohris/absences",
        "json": json_json_body,
        "params": params,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[List["Absence"]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = Absence.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[List["Absence"]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    json_body: List[str],
    details: Union[Unset, None, GetAbsencesAgentDetails] = UNSET,
    include_managements: Union[Unset, None, bool] = UNSET,
) -> Response[List["Absence"]]:
    """Extrait les informations des absences

    Args:
        details (Union[Unset, None, GetAbsencesAgentDetails]):
        include_managements (Union[Unset, None, bool]):
        json_body (List[str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[List['Absence']]
    """

    kwargs = _get_kwargs(
        json_body=json_body,
        details=details,
        include_managements=include_managements,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    json_body: List[str],
    details: Union[Unset, None, GetAbsencesAgentDetails] = UNSET,
    include_managements: Union[Unset, None, bool] = UNSET,
) -> Optional[List["Absence"]]:
    """Extrait les informations des absences

    Args:
        details (Union[Unset, None, GetAbsencesAgentDetails]):
        include_managements (Union[Unset, None, bool]):
        json_body (List[str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        List['Absence']
    """

    return sync_detailed(
        client=client,
        json_body=json_body,
        details=details,
        include_managements=include_managements,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    json_body: List[str],
    details: Union[Unset, None, GetAbsencesAgentDetails] = UNSET,
    include_managements: Union[Unset, None, bool] = UNSET,
) -> Response[List["Absence"]]:
    """Extrait les informations des absences

    Args:
        details (Union[Unset, None, GetAbsencesAgentDetails]):
        include_managements (Union[Unset, None, bool]):
        json_body (List[str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[List['Absence']]
    """

    kwargs = _get_kwargs(
        json_body=json_body,
        details=details,
        include_managements=include_managements,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    json_body: List[str],
    details: Union[Unset, None, GetAbsencesAgentDetails] = UNSET,
    include_managements: Union[Unset, None, bool] = UNSET,
) -> Optional[List["Absence"]]:
    """Extrait les informations des absences

    Args:
        details (Union[Unset, None, GetAbsencesAgentDetails]):
        include_managements (Union[Unset, None, bool]):
        json_body (List[str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        List['Absence']
    """

    return (
        await asyncio_detailed(
            client=client,
            json_body=json_body,
            details=details,
            include_managements=include_managements,
        )
    ).parsed
