from http import HTTPStatus
from typing import Any, Dict, Optional, Union, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.get_correspondence_type import GetCorrespondenceType
from ...types import Response


def _get_kwargs(
    type: GetCorrespondenceType,
    value: str,
) -> Dict[str, Any]:
    pass

    return {
        "method": "get",
        "url": "/ohris/correspondences/{type}/{value}".format(
            type=type,
            value=value,
        ),
    }


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[str]:
    if response.status_code == HTTPStatus.OK:
        response_200 = cast(str, response.json())
        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[str]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    type: GetCorrespondenceType,
    value: str,
    *,
    client: AuthenticatedClient,
) -> Response[str]:
    """Récupère la correspondance SIHAM/Ohris d'une valeur ciblée

    Args:
        type (GetCorrespondenceType):
        value (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[str]
    """

    kwargs = _get_kwargs(
        type=type,
        value=value,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    type: GetCorrespondenceType,
    value: str,
    *,
    client: AuthenticatedClient,
) -> Optional[str]:
    """Récupère la correspondance SIHAM/Ohris d'une valeur ciblée

    Args:
        type (GetCorrespondenceType):
        value (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        str
    """

    return sync_detailed(
        type=type,
        value=value,
        client=client,
    ).parsed


async def asyncio_detailed(
    type: GetCorrespondenceType,
    value: str,
    *,
    client: AuthenticatedClient,
) -> Response[str]:
    """Récupère la correspondance SIHAM/Ohris d'une valeur ciblée

    Args:
        type (GetCorrespondenceType):
        value (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[str]
    """

    kwargs = _get_kwargs(
        type=type,
        value=value,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    type: GetCorrespondenceType,
    value: str,
    *,
    client: AuthenticatedClient,
) -> Optional[str]:
    """Récupère la correspondance SIHAM/Ohris d'une valeur ciblée

    Args:
        type (GetCorrespondenceType):
        value (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        str
    """

    return (
        await asyncio_detailed(
            type=type,
            value=value,
            client=client,
        )
    ).parsed
