from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.get_correspondences_type import GetCorrespondencesType
from ...types import Response


def _get_kwargs(
    type: GetCorrespondencesType,
) -> Dict[str, Any]:
    pass

    return {
        "method": "get",
        "url": "/ohris/correspondences/{type}".format(
            type=type,
        ),
    }


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[List[str]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = cast(List[str], response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[List[str]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    type: GetCorrespondencesType,
    *,
    client: AuthenticatedClient,
) -> Response[List[str]]:
    """Récupère un tableau de correspondances SIHAM/Ohris

    Args:
        type (GetCorrespondencesType):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[List[str]]
    """

    kwargs = _get_kwargs(
        type=type,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    type: GetCorrespondencesType,
    *,
    client: AuthenticatedClient,
) -> Optional[List[str]]:
    """Récupère un tableau de correspondances SIHAM/Ohris

    Args:
        type (GetCorrespondencesType):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        List[str]
    """

    return sync_detailed(
        type=type,
        client=client,
    ).parsed


async def asyncio_detailed(
    type: GetCorrespondencesType,
    *,
    client: AuthenticatedClient,
) -> Response[List[str]]:
    """Récupère un tableau de correspondances SIHAM/Ohris

    Args:
        type (GetCorrespondencesType):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[List[str]]
    """

    kwargs = _get_kwargs(
        type=type,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    type: GetCorrespondencesType,
    *,
    client: AuthenticatedClient,
) -> Optional[List[str]]:
    """Récupère un tableau de correspondances SIHAM/Ohris

    Args:
        type (GetCorrespondencesType):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        List[str]
    """

    return (
        await asyncio_detailed(
            type=type,
            client=client,
        )
    ).parsed
