from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    statuses: List[str],
    start: Union[Unset, None, str] = UNSET,
    end: Union[Unset, None, str] = UNSET,
) -> Dict[str, Any]:
    pass

    params: Dict[str, Any] = {}
    json_statuses = statuses

    params["statuses"] = json_statuses

    params["start"] = start

    params["end"] = end

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": "/situations/statuses",
        "params": params,
    }


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[List[str]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = cast(List[str], response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[List[str]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    statuses: List[str],
    start: Union[Unset, None, str] = UNSET,
    end: Union[Unset, None, str] = UNSET,
) -> Response[List[str]]:
    """Récupère la liste des numéros de dossiers liés à des statuts de situations administratives (Profil
    Individuel de Paie)

    Args:
        statuses (List[str]):
        start (Union[Unset, None, str]):
        end (Union[Unset, None, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[List[str]]
    """

    kwargs = _get_kwargs(
        statuses=statuses,
        start=start,
        end=end,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    statuses: List[str],
    start: Union[Unset, None, str] = UNSET,
    end: Union[Unset, None, str] = UNSET,
) -> Optional[List[str]]:
    """Récupère la liste des numéros de dossiers liés à des statuts de situations administratives (Profil
    Individuel de Paie)

    Args:
        statuses (List[str]):
        start (Union[Unset, None, str]):
        end (Union[Unset, None, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        List[str]
    """

    return sync_detailed(
        client=client,
        statuses=statuses,
        start=start,
        end=end,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    statuses: List[str],
    start: Union[Unset, None, str] = UNSET,
    end: Union[Unset, None, str] = UNSET,
) -> Response[List[str]]:
    """Récupère la liste des numéros de dossiers liés à des statuts de situations administratives (Profil
    Individuel de Paie)

    Args:
        statuses (List[str]):
        start (Union[Unset, None, str]):
        end (Union[Unset, None, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[List[str]]
    """

    kwargs = _get_kwargs(
        statuses=statuses,
        start=start,
        end=end,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    statuses: List[str],
    start: Union[Unset, None, str] = UNSET,
    end: Union[Unset, None, str] = UNSET,
) -> Optional[List[str]]:
    """Récupère la liste des numéros de dossiers liés à des statuts de situations administratives (Profil
    Individuel de Paie)

    Args:
        statuses (List[str]):
        start (Union[Unset, None, str]):
        end (Union[Unset, None, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        List[str]
    """

    return (
        await asyncio_detailed(
            client=client,
            statuses=statuses,
            start=start,
            end=end,
        )
    ).parsed
