import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.label import Label


T = TypeVar("T", bound="Absence")


@_attrs_define
class Absence:
    """Informations minimales sur une basence

    Attributes:
        matricule (Union[Unset, str]): Numéro de dossier SIHAM
        start (Union[Unset, datetime.datetime]): Date de début
        start_midday (Union[Unset, bool]): Démarre à midi
        end (Union[Unset, datetime.datetime]): Date de fin
        end_midday (Union[Unset, bool]): Finit à midi
        is_planned (Union[Unset, bool]): Fin prévisionnelle
        is_management (Union[Unset, bool]): Détermine s'il s'agit d'une absence à des fins de gestion
        is_extension (Union[Unset, bool]): Détermine s'il s'agit d'une prologation
        is_abroad (Union[Unset, bool]): Détermine s'il s'agit d'une absence à l'étranger
        type (Union[Unset, Label]): Etiquette
        validity (Union[Unset, int]): Témoin de validité des informaitons (4 = valide)
        is_pending (Union[Unset, bool]): Détermine si cette absence n'est pas encore active
        is_expired (Union[Unset, bool]): Détermine si cette absence est expirée
        is_active (Union[Unset, bool]): Détermine si cette absence est active
    """

    matricule: Union[Unset, str] = UNSET
    start: Union[Unset, datetime.datetime] = UNSET
    start_midday: Union[Unset, bool] = UNSET
    end: Union[Unset, datetime.datetime] = UNSET
    end_midday: Union[Unset, bool] = UNSET
    is_planned: Union[Unset, bool] = UNSET
    is_management: Union[Unset, bool] = UNSET
    is_extension: Union[Unset, bool] = UNSET
    is_abroad: Union[Unset, bool] = UNSET
    type: Union[Unset, "Label"] = UNSET
    validity: Union[Unset, int] = UNSET
    is_pending: Union[Unset, bool] = UNSET
    is_expired: Union[Unset, bool] = UNSET
    is_active: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        matricule = self.matricule
        start: Union[Unset, str] = UNSET
        if not isinstance(self.start, Unset):
            start = self.start.isoformat()

        start_midday = self.start_midday
        end: Union[Unset, str] = UNSET
        if not isinstance(self.end, Unset):
            end = self.end.isoformat()

        end_midday = self.end_midday
        is_planned = self.is_planned
        is_management = self.is_management
        is_extension = self.is_extension
        is_abroad = self.is_abroad
        type: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.to_dict()

        validity = self.validity
        is_pending = self.is_pending
        is_expired = self.is_expired
        is_active = self.is_active

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if matricule is not UNSET:
            field_dict["matricule"] = matricule
        if start is not UNSET:
            field_dict["start"] = start
        if start_midday is not UNSET:
            field_dict["startMidday"] = start_midday
        if end is not UNSET:
            field_dict["end"] = end
        if end_midday is not UNSET:
            field_dict["endMidday"] = end_midday
        if is_planned is not UNSET:
            field_dict["isPlanned"] = is_planned
        if is_management is not UNSET:
            field_dict["isManagement"] = is_management
        if is_extension is not UNSET:
            field_dict["isExtension"] = is_extension
        if is_abroad is not UNSET:
            field_dict["isAbroad"] = is_abroad
        if type is not UNSET:
            field_dict["type"] = type
        if validity is not UNSET:
            field_dict["validity"] = validity
        if is_pending is not UNSET:
            field_dict["isPending"] = is_pending
        if is_expired is not UNSET:
            field_dict["isExpired"] = is_expired
        if is_active is not UNSET:
            field_dict["isActive"] = is_active

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.label import Label

        if not src_dict:
            return None
        d = src_dict.copy()
        matricule = d.pop("matricule", UNSET)

        _start = d.pop("start", UNSET)
        start: Union[Unset, datetime.datetime]
        if isinstance(_start, Unset):
            start = UNSET
        else:
            start = isoparse(_start)

        start_midday = d.pop("startMidday", UNSET)

        _end = d.pop("end", UNSET)
        end: Union[Unset, datetime.datetime]
        if isinstance(_end, Unset):
            end = UNSET
        else:
            end = isoparse(_end)

        end_midday = d.pop("endMidday", UNSET)

        is_planned = d.pop("isPlanned", UNSET)

        is_management = d.pop("isManagement", UNSET)

        is_extension = d.pop("isExtension", UNSET)

        is_abroad = d.pop("isAbroad", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, Label]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = Label.from_dict(_type)

        validity = d.pop("validity", UNSET)

        is_pending = d.pop("isPending", UNSET)

        is_expired = d.pop("isExpired", UNSET)

        is_active = d.pop("isActive", UNSET)

        absence = cls(
            matricule=matricule,
            start=start,
            start_midday=start_midday,
            end=end,
            end_midday=end_midday,
            is_planned=is_planned,
            is_management=is_management,
            is_extension=is_extension,
            is_abroad=is_abroad,
            type=type,
            validity=validity,
            is_pending=is_pending,
            is_expired=is_expired,
            is_active=is_active,
        )

        absence.additional_properties = d
        return absence

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
