from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.career import Career
    from ..models.complementary_activity import ComplementaryActivity
    from ..models.contract import Contract
    from ..models.functional_assignment import FunctionalAssignment
    from ..models.hierarchical_assignment import HierarchicalAssignment
    from ..models.modification_details import ModificationDetails
    from ..models.personal_details_lite import PersonalDetailsLite
    from ..models.situation_population import SituationPopulation
    from ..models.situation_status import SituationStatus
    from ..models.speciality import Speciality


T = TypeVar("T", bound="Agent")


@_attrs_define
class Agent:
    """Informations générales sur un agent

    Attributes:
        identity (Union[Unset, PersonalDetailsLite]): Version simplifiée des informations personnelles d'un agent
        hierarchical_assignments (Union[Unset, List['HierarchicalAssignment']]): Collection des affectations
            hiérarchiques
        functional_assignments (Union[Unset, List['FunctionalAssignment']]): Collection des affectations fonctionnelles
        contracts (Union[Unset, List['Contract']]): Collection des contrats
        complementary_activities (Union[Unset, List['ComplementaryActivity']]): Activités complémentaires
        careers (Union[Unset, List['Career']]): Collection des informations de carrières
        populations (Union[Unset, List['SituationPopulation']]): Collection des informations liées aux populations
        statuses (Union[Unset, List['SituationStatus']]): Collection des informations liées aux statuts des situations
            administratives (Profils Individuels de Paie)
        specialities (Union[Unset, List['Speciality']]): Collection des informations liées aux spécialités
        modifications (Union[Unset, List['ModificationDetails']]): Collection des informations de modification du
            dossier
    """

    identity: Union[Unset, "PersonalDetailsLite"] = UNSET
    hierarchical_assignments: Union[Unset, List["HierarchicalAssignment"]] = UNSET
    functional_assignments: Union[Unset, List["FunctionalAssignment"]] = UNSET
    contracts: Union[Unset, List["Contract"]] = UNSET
    complementary_activities: Union[Unset, List["ComplementaryActivity"]] = UNSET
    careers: Union[Unset, List["Career"]] = UNSET
    populations: Union[Unset, List["SituationPopulation"]] = UNSET
    statuses: Union[Unset, List["SituationStatus"]] = UNSET
    specialities: Union[Unset, List["Speciality"]] = UNSET
    modifications: Union[Unset, List["ModificationDetails"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        identity: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.identity, Unset):
            identity = self.identity.to_dict()

        hierarchical_assignments: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.hierarchical_assignments, Unset):
            hierarchical_assignments = []
            for hierarchical_assignments_item_data in self.hierarchical_assignments:
                hierarchical_assignments_item = hierarchical_assignments_item_data.to_dict()

                hierarchical_assignments.append(hierarchical_assignments_item)

        functional_assignments: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.functional_assignments, Unset):
            functional_assignments = []
            for functional_assignments_item_data in self.functional_assignments:
                functional_assignments_item = functional_assignments_item_data.to_dict()

                functional_assignments.append(functional_assignments_item)

        contracts: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.contracts, Unset):
            contracts = []
            for contracts_item_data in self.contracts:
                contracts_item = contracts_item_data.to_dict()

                contracts.append(contracts_item)

        complementary_activities: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.complementary_activities, Unset):
            complementary_activities = []
            for complementary_activities_item_data in self.complementary_activities:
                complementary_activities_item = complementary_activities_item_data.to_dict()

                complementary_activities.append(complementary_activities_item)

        careers: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.careers, Unset):
            careers = []
            for careers_item_data in self.careers:
                careers_item = careers_item_data.to_dict()

                careers.append(careers_item)

        populations: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.populations, Unset):
            populations = []
            for populations_item_data in self.populations:
                populations_item = populations_item_data.to_dict()

                populations.append(populations_item)

        statuses: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.statuses, Unset):
            statuses = []
            for statuses_item_data in self.statuses:
                statuses_item = statuses_item_data.to_dict()

                statuses.append(statuses_item)

        specialities: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.specialities, Unset):
            specialities = []
            for specialities_item_data in self.specialities:
                specialities_item = specialities_item_data.to_dict()

                specialities.append(specialities_item)

        modifications: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.modifications, Unset):
            modifications = []
            for modifications_item_data in self.modifications:
                modifications_item = modifications_item_data.to_dict()

                modifications.append(modifications_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if identity is not UNSET:
            field_dict["identity"] = identity
        if hierarchical_assignments is not UNSET:
            field_dict["hierarchicalAssignments"] = hierarchical_assignments
        if functional_assignments is not UNSET:
            field_dict["functionalAssignments"] = functional_assignments
        if contracts is not UNSET:
            field_dict["contracts"] = contracts
        if complementary_activities is not UNSET:
            field_dict["complementaryActivities"] = complementary_activities
        if careers is not UNSET:
            field_dict["careers"] = careers
        if populations is not UNSET:
            field_dict["populations"] = populations
        if statuses is not UNSET:
            field_dict["statuses"] = statuses
        if specialities is not UNSET:
            field_dict["specialities"] = specialities
        if modifications is not UNSET:
            field_dict["modifications"] = modifications

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.career import Career
        from ..models.complementary_activity import ComplementaryActivity
        from ..models.contract import Contract
        from ..models.functional_assignment import FunctionalAssignment
        from ..models.hierarchical_assignment import HierarchicalAssignment
        from ..models.modification_details import ModificationDetails
        from ..models.personal_details_lite import PersonalDetailsLite
        from ..models.situation_population import SituationPopulation
        from ..models.situation_status import SituationStatus
        from ..models.speciality import Speciality

        if not src_dict:
            return None
        d = src_dict.copy()
        _identity = d.pop("identity", UNSET)
        identity: Union[Unset, PersonalDetailsLite]
        if isinstance(_identity, Unset):
            identity = UNSET
        else:
            identity = PersonalDetailsLite.from_dict(_identity)

        hierarchical_assignments = []
        _hierarchical_assignments = d.pop("hierarchicalAssignments", UNSET)
        for hierarchical_assignments_item_data in _hierarchical_assignments or []:
            hierarchical_assignments_item = HierarchicalAssignment.from_dict(hierarchical_assignments_item_data)

            hierarchical_assignments.append(hierarchical_assignments_item)

        functional_assignments = []
        _functional_assignments = d.pop("functionalAssignments", UNSET)
        for functional_assignments_item_data in _functional_assignments or []:
            functional_assignments_item = FunctionalAssignment.from_dict(functional_assignments_item_data)

            functional_assignments.append(functional_assignments_item)

        contracts = []
        _contracts = d.pop("contracts", UNSET)
        for contracts_item_data in _contracts or []:
            contracts_item = Contract.from_dict(contracts_item_data)

            contracts.append(contracts_item)

        complementary_activities = []
        _complementary_activities = d.pop("complementaryActivities", UNSET)
        for complementary_activities_item_data in _complementary_activities or []:
            complementary_activities_item = ComplementaryActivity.from_dict(complementary_activities_item_data)

            complementary_activities.append(complementary_activities_item)

        careers = []
        _careers = d.pop("careers", UNSET)
        for careers_item_data in _careers or []:
            careers_item = Career.from_dict(careers_item_data)

            careers.append(careers_item)

        populations = []
        _populations = d.pop("populations", UNSET)
        for populations_item_data in _populations or []:
            populations_item = SituationPopulation.from_dict(populations_item_data)

            populations.append(populations_item)

        statuses = []
        _statuses = d.pop("statuses", UNSET)
        for statuses_item_data in _statuses or []:
            statuses_item = SituationStatus.from_dict(statuses_item_data)

            statuses.append(statuses_item)

        specialities = []
        _specialities = d.pop("specialities", UNSET)
        for specialities_item_data in _specialities or []:
            specialities_item = Speciality.from_dict(specialities_item_data)

            specialities.append(specialities_item)

        modifications = []
        _modifications = d.pop("modifications", UNSET)
        for modifications_item_data in _modifications or []:
            modifications_item = ModificationDetails.from_dict(modifications_item_data)

            modifications.append(modifications_item)

        agent = cls(
            identity=identity,
            hierarchical_assignments=hierarchical_assignments,
            functional_assignments=functional_assignments,
            contracts=contracts,
            complementary_activities=complementary_activities,
            careers=careers,
            populations=populations,
            statuses=statuses,
            specialities=specialities,
            modifications=modifications,
        )

        agent.additional_properties = d
        return agent

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
