from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.hierarchical_assignment import HierarchicalAssignment
    from ..models.personal_details_lite import PersonalDetailsLite


T = TypeVar("T", bound="AgentLite")


@_attrs_define
class AgentLite:
    """Informations minimales sur un agent

    Attributes:
        identity (Union[Unset, PersonalDetailsLite]): Version simplifiée des informations personnelles d'un agent
        assignemnt (Union[Unset, HierarchicalAssignment]): Affectation principale (hiérarchique) d'un agent
    """

    identity: Union[Unset, "PersonalDetailsLite"] = UNSET
    assignemnt: Union[Unset, "HierarchicalAssignment"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        identity: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.identity, Unset):
            identity = self.identity.to_dict()

        assignemnt: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.assignemnt, Unset):
            assignemnt = self.assignemnt.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if identity is not UNSET:
            field_dict["identity"] = identity
        if assignemnt is not UNSET:
            field_dict["assignemnt"] = assignemnt

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.hierarchical_assignment import HierarchicalAssignment
        from ..models.personal_details_lite import PersonalDetailsLite

        if not src_dict:
            return None
        d = src_dict.copy()
        _identity = d.pop("identity", UNSET)
        identity: Union[Unset, PersonalDetailsLite]
        if isinstance(_identity, Unset):
            identity = UNSET
        else:
            identity = PersonalDetailsLite.from_dict(_identity)

        _assignemnt = d.pop("assignemnt", UNSET)
        assignemnt: Union[Unset, HierarchicalAssignment]
        if isinstance(_assignemnt, Unset):
            assignemnt = UNSET
        else:
            assignemnt = HierarchicalAssignment.from_dict(_assignemnt)

        agent_lite = cls(
            identity=identity,
            assignemnt=assignemnt,
        )

        agent_lite.additional_properties = d
        return agent_lite

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
