from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.agent_lite import AgentLite
    from ..models.permission import Permission


T = TypeVar("T", bound="AgentPermission")


@_attrs_define
class AgentPermission:
    """
    Attributes:
        agent (Union[Unset, AgentLite]): Informations minimales sur un agent
        permissions (Union[Unset, List['Permission']]): Collection de permissions
    """

    agent: Union[Unset, "AgentLite"] = UNSET
    permissions: Union[Unset, List["Permission"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        agent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.agent, Unset):
            agent = self.agent.to_dict()

        permissions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.permissions, Unset):
            permissions = []
            for permissions_item_data in self.permissions:
                permissions_item = permissions_item_data.to_dict()

                permissions.append(permissions_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if agent is not UNSET:
            field_dict["agent"] = agent
        if permissions is not UNSET:
            field_dict["permissions"] = permissions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.agent_lite import AgentLite
        from ..models.permission import Permission

        if not src_dict:
            return None
        d = src_dict.copy()
        _agent = d.pop("agent", UNSET)
        agent: Union[Unset, AgentLite]
        if isinstance(_agent, Unset):
            agent = UNSET
        else:
            agent = AgentLite.from_dict(_agent)

        permissions = []
        _permissions = d.pop("permissions", UNSET)
        for permissions_item_data in _permissions or []:
            permissions_item = Permission.from_dict(permissions_item_data)

            permissions.append(permissions_item)

        agent_permission = cls(
            agent=agent,
            permissions=permissions,
        )

        agent_permission.additional_properties = d
        return agent_permission

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
