import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="AgentPreview")


@_attrs_define
class AgentPreview:
    """
    Attributes:
        matricule (Union[Unset, str]): Numéro de dossier SIHAM
        assignment_code (Union[Unset, str]): Code identifiant de l'Unité d'Organisation de l'affectation hierarchique
        status (Union[Unset, str]): Statut
        compensation_index (Union[Unset, int]): Indice de rénumération
        start (Union[Unset, datetime.datetime]): Date de début d'activité
        end (Union[Unset, datetime.datetime]): Date de fin d'activité
    """

    matricule: Union[Unset, str] = UNSET
    assignment_code: Union[Unset, str] = UNSET
    status: Union[Unset, str] = UNSET
    compensation_index: Union[Unset, int] = UNSET
    start: Union[Unset, datetime.datetime] = UNSET
    end: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        matricule = self.matricule
        assignment_code = self.assignment_code
        status = self.status
        compensation_index = self.compensation_index
        start: Union[Unset, str] = UNSET
        if not isinstance(self.start, Unset):
            start = self.start.isoformat()

        end: Union[Unset, str] = UNSET
        if not isinstance(self.end, Unset):
            end = self.end.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if matricule is not UNSET:
            field_dict["matricule"] = matricule
        if assignment_code is not UNSET:
            field_dict["assignmentCode"] = assignment_code
        if status is not UNSET:
            field_dict["status"] = status
        if compensation_index is not UNSET:
            field_dict["compensationIndex"] = compensation_index
        if start is not UNSET:
            field_dict["start"] = start
        if end is not UNSET:
            field_dict["end"] = end

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        if not src_dict:
            return None
        d = src_dict.copy()
        matricule = d.pop("matricule", UNSET)

        assignment_code = d.pop("assignmentCode", UNSET)

        status = d.pop("status", UNSET)

        compensation_index = d.pop("compensationIndex", UNSET)

        _start = d.pop("start", UNSET)
        start: Union[Unset, datetime.datetime]
        if isinstance(_start, Unset):
            start = UNSET
        else:
            start = isoparse(_start)

        _end = d.pop("end", UNSET)
        end: Union[Unset, datetime.datetime]
        if isinstance(_end, Unset):
            end = UNSET
        else:
            end = isoparse(_end)

        agent_preview = cls(
            matricule=matricule,
            assignment_code=assignment_code,
            status=status,
            compensation_index=compensation_index,
            start=start,
            end=end,
        )

        agent_preview.additional_properties = d
        return agent_preview

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
