import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.connector_status import ConnectorStatus


T = TypeVar("T", bound="ApplicationStatus")


@_attrs_define
class ApplicationStatus:
    """Informations sur le statut global de l'application

    Attributes:
        name (Union[Unset, str]): Nom de l'application
        artifact_id (Union[Unset, str]): ArtifactId
        description (Union[Unset, str]): Description
        url (Union[Unset, str]): URL
        version (Union[Unset, str]): Version
        starttime (Union[Unset, datetime.datetime]): Date de démarrage
        time (Union[Unset, datetime.datetime]): Date du relevé d'état
        labels_updated (Union[Unset, datetime.datetime]): Date de mise à jour du cache des étiquettes
        job_types_updated (Union[Unset, datetime.datetime]): Date de mise à jour du cache des emplois types
        disciplines_updated (Union[Unset, datetime.datetime]): Date de mise à jour du cache des disciplines
        organizational_units_updated (Union[Unset, datetime.datetime]): Date de mise à jour du cache des unités
            organisationelles
        status (Union[Unset, str]): Etat global
        outbound_connections_statuses (Union[Unset, List['ConnectorStatus']]): Liste des états des connecteurs utilisés
    """

    name: Union[Unset, str] = UNSET
    artifact_id: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    url: Union[Unset, str] = UNSET
    version: Union[Unset, str] = UNSET
    starttime: Union[Unset, datetime.datetime] = UNSET
    time: Union[Unset, datetime.datetime] = UNSET
    labels_updated: Union[Unset, datetime.datetime] = UNSET
    job_types_updated: Union[Unset, datetime.datetime] = UNSET
    disciplines_updated: Union[Unset, datetime.datetime] = UNSET
    organizational_units_updated: Union[Unset, datetime.datetime] = UNSET
    status: Union[Unset, str] = UNSET
    outbound_connections_statuses: Union[Unset, List["ConnectorStatus"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        artifact_id = self.artifact_id
        description = self.description
        url = self.url
        version = self.version
        starttime: Union[Unset, str] = UNSET
        if not isinstance(self.starttime, Unset):
            starttime = self.starttime.isoformat()

        time: Union[Unset, str] = UNSET
        if not isinstance(self.time, Unset):
            time = self.time.isoformat()

        labels_updated: Union[Unset, str] = UNSET
        if not isinstance(self.labels_updated, Unset):
            labels_updated = self.labels_updated.isoformat()

        job_types_updated: Union[Unset, str] = UNSET
        if not isinstance(self.job_types_updated, Unset):
            job_types_updated = self.job_types_updated.isoformat()

        disciplines_updated: Union[Unset, str] = UNSET
        if not isinstance(self.disciplines_updated, Unset):
            disciplines_updated = self.disciplines_updated.isoformat()

        organizational_units_updated: Union[Unset, str] = UNSET
        if not isinstance(self.organizational_units_updated, Unset):
            organizational_units_updated = self.organizational_units_updated.isoformat()

        status = self.status
        outbound_connections_statuses: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.outbound_connections_statuses, Unset):
            outbound_connections_statuses = []
            for outbound_connections_statuses_item_data in self.outbound_connections_statuses:
                outbound_connections_statuses_item = outbound_connections_statuses_item_data.to_dict()

                outbound_connections_statuses.append(outbound_connections_statuses_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if artifact_id is not UNSET:
            field_dict["artifactId"] = artifact_id
        if description is not UNSET:
            field_dict["description"] = description
        if url is not UNSET:
            field_dict["url"] = url
        if version is not UNSET:
            field_dict["version"] = version
        if starttime is not UNSET:
            field_dict["starttime"] = starttime
        if time is not UNSET:
            field_dict["time"] = time
        if labels_updated is not UNSET:
            field_dict["labelsUpdated"] = labels_updated
        if job_types_updated is not UNSET:
            field_dict["jobTypesUpdated"] = job_types_updated
        if disciplines_updated is not UNSET:
            field_dict["disciplinesUpdated"] = disciplines_updated
        if organizational_units_updated is not UNSET:
            field_dict["organizationalUnitsUpdated"] = organizational_units_updated
        if status is not UNSET:
            field_dict["status"] = status
        if outbound_connections_statuses is not UNSET:
            field_dict["outboundConnectionsStatuses"] = outbound_connections_statuses

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.connector_status import ConnectorStatus

        if not src_dict:
            return None
        d = src_dict.copy()
        name = d.pop("name", UNSET)

        artifact_id = d.pop("artifactId", UNSET)

        description = d.pop("description", UNSET)

        url = d.pop("url", UNSET)

        version = d.pop("version", UNSET)

        _starttime = d.pop("starttime", UNSET)
        starttime: Union[Unset, datetime.datetime]
        if isinstance(_starttime, Unset):
            starttime = UNSET
        else:
            starttime = isoparse(_starttime)

        _time = d.pop("time", UNSET)
        time: Union[Unset, datetime.datetime]
        if isinstance(_time, Unset):
            time = UNSET
        else:
            time = isoparse(_time)

        _labels_updated = d.pop("labelsUpdated", UNSET)
        labels_updated: Union[Unset, datetime.datetime]
        if isinstance(_labels_updated, Unset):
            labels_updated = UNSET
        else:
            labels_updated = isoparse(_labels_updated)

        _job_types_updated = d.pop("jobTypesUpdated", UNSET)
        job_types_updated: Union[Unset, datetime.datetime]
        if isinstance(_job_types_updated, Unset):
            job_types_updated = UNSET
        else:
            job_types_updated = isoparse(_job_types_updated)

        _disciplines_updated = d.pop("disciplinesUpdated", UNSET)
        disciplines_updated: Union[Unset, datetime.datetime]
        if isinstance(_disciplines_updated, Unset):
            disciplines_updated = UNSET
        else:
            disciplines_updated = isoparse(_disciplines_updated)

        _organizational_units_updated = d.pop("organizationalUnitsUpdated", UNSET)
        organizational_units_updated: Union[Unset, datetime.datetime]
        if isinstance(_organizational_units_updated, Unset):
            organizational_units_updated = UNSET
        else:
            organizational_units_updated = isoparse(_organizational_units_updated)

        status = d.pop("status", UNSET)

        outbound_connections_statuses = []
        _outbound_connections_statuses = d.pop("outboundConnectionsStatuses", UNSET)
        for outbound_connections_statuses_item_data in _outbound_connections_statuses or []:
            outbound_connections_statuses_item = ConnectorStatus.from_dict(outbound_connections_statuses_item_data)

            outbound_connections_statuses.append(outbound_connections_statuses_item)

        application_status = cls(
            name=name,
            artifact_id=artifact_id,
            description=description,
            url=url,
            version=version,
            starttime=starttime,
            time=time,
            labels_updated=labels_updated,
            job_types_updated=job_types_updated,
            disciplines_updated=disciplines_updated,
            organizational_units_updated=organizational_units_updated,
            status=status,
            outbound_connections_statuses=outbound_connections_statuses,
        )

        application_status.additional_properties = d
        return application_status

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
