import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.label import Label


T = TypeVar("T", bound="Career")


@_attrs_define
class Career:
    """Carrière d'un agent

    Attributes:
        matricule (Union[Unset, str]): Numéro de dossier
        start_effective (Union[Unset, datetime.datetime]): Date de début
        end_effective (Union[Unset, datetime.datetime]): Date de fin
        decree (Union[Unset, str]): Numéro d'arrêté
        order (Union[Unset, int]): Numéro d'ordre
        line (Union[Unset, int]): Numéro de ligne
        type (Union[Unset, str]): Type
        validity (Union[Unset, int]): Témoin de validité des informaitons (4 = valide)
        status (Union[Unset, Label]): Etiquette
        hierarchical_group (Union[Unset, Label]): Etiquette
        career_scale (Union[Unset, Label]): Etiquette
        job_category (Union[Unset, Label]): Etiquette
        job_group (Union[Unset, Label]): Etiquette
        job_group_start (Union[Unset, datetime.datetime]): Date d'entrée dans le corps
        career_grade (Union[Unset, Label]): Etiquette
        career_grade_start (Union[Unset, datetime.datetime]): Date d'entrée dans le grade
        tg_grade (Union[Unset, Label]): Etiquette
        is_hosted (Union[Unset, bool]): Témoin d'hébergement
        is_teacher_or_reasearcher (Union[Unset, bool]): Témoin d'activité de recherche ou d'enseignement
        is_staff (Union[Unset, bool]): Témoin d'activités techniques ou administratives
        is_teacher (Union[Unset, bool]): Témoin d'activités d'enseignement
        is_researcher (Union[Unset, bool]): Témoin d'activités de recherche
        is_secondary_school_teacher (Union[Unset, bool]): Témoin d'enseignant du second degré
        is_active (Union[Unset, bool]): Témoin d'activité
        categories (Union[Unset, List[str]]): Catégorisation de la carrière
        is_pending (Union[Unset, bool]): Témoin de mise en attente (pas encore commencé)
        is_expired (Union[Unset, bool]): Témoin d'expiration
    """

    matricule: Union[Unset, str] = UNSET
    start_effective: Union[Unset, datetime.datetime] = UNSET
    end_effective: Union[Unset, datetime.datetime] = UNSET
    decree: Union[Unset, str] = UNSET
    order: Union[Unset, int] = UNSET
    line: Union[Unset, int] = UNSET
    type: Union[Unset, str] = UNSET
    validity: Union[Unset, int] = UNSET
    status: Union[Unset, "Label"] = UNSET
    hierarchical_group: Union[Unset, "Label"] = UNSET
    career_scale: Union[Unset, "Label"] = UNSET
    job_category: Union[Unset, "Label"] = UNSET
    job_group: Union[Unset, "Label"] = UNSET
    job_group_start: Union[Unset, datetime.datetime] = UNSET
    career_grade: Union[Unset, "Label"] = UNSET
    career_grade_start: Union[Unset, datetime.datetime] = UNSET
    tg_grade: Union[Unset, "Label"] = UNSET
    is_hosted: Union[Unset, bool] = UNSET
    is_teacher_or_reasearcher: Union[Unset, bool] = UNSET
    is_staff: Union[Unset, bool] = UNSET
    is_teacher: Union[Unset, bool] = UNSET
    is_researcher: Union[Unset, bool] = UNSET
    is_secondary_school_teacher: Union[Unset, bool] = UNSET
    is_active: Union[Unset, bool] = UNSET
    categories: Union[Unset, List[str]] = UNSET
    is_pending: Union[Unset, bool] = UNSET
    is_expired: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        matricule = self.matricule
        start_effective: Union[Unset, str] = UNSET
        if not isinstance(self.start_effective, Unset):
            start_effective = self.start_effective.isoformat()

        end_effective: Union[Unset, str] = UNSET
        if not isinstance(self.end_effective, Unset):
            end_effective = self.end_effective.isoformat()

        decree = self.decree
        order = self.order
        line = self.line
        type = self.type
        validity = self.validity
        status: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.to_dict()

        hierarchical_group: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.hierarchical_group, Unset):
            hierarchical_group = self.hierarchical_group.to_dict()

        career_scale: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.career_scale, Unset):
            career_scale = self.career_scale.to_dict()

        job_category: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.job_category, Unset):
            job_category = self.job_category.to_dict()

        job_group: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.job_group, Unset):
            job_group = self.job_group.to_dict()

        job_group_start: Union[Unset, str] = UNSET
        if not isinstance(self.job_group_start, Unset):
            job_group_start = self.job_group_start.isoformat()

        career_grade: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.career_grade, Unset):
            career_grade = self.career_grade.to_dict()

        career_grade_start: Union[Unset, str] = UNSET
        if not isinstance(self.career_grade_start, Unset):
            career_grade_start = self.career_grade_start.isoformat()

        tg_grade: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.tg_grade, Unset):
            tg_grade = self.tg_grade.to_dict()

        is_hosted = self.is_hosted
        is_teacher_or_reasearcher = self.is_teacher_or_reasearcher
        is_staff = self.is_staff
        is_teacher = self.is_teacher
        is_researcher = self.is_researcher
        is_secondary_school_teacher = self.is_secondary_school_teacher
        is_active = self.is_active
        categories: Union[Unset, List[str]] = UNSET
        if not isinstance(self.categories, Unset):
            categories = self.categories

        is_pending = self.is_pending
        is_expired = self.is_expired

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if matricule is not UNSET:
            field_dict["matricule"] = matricule
        if start_effective is not UNSET:
            field_dict["startEffective"] = start_effective
        if end_effective is not UNSET:
            field_dict["endEffective"] = end_effective
        if decree is not UNSET:
            field_dict["decree"] = decree
        if order is not UNSET:
            field_dict["order"] = order
        if line is not UNSET:
            field_dict["line"] = line
        if type is not UNSET:
            field_dict["type"] = type
        if validity is not UNSET:
            field_dict["validity"] = validity
        if status is not UNSET:
            field_dict["status"] = status
        if hierarchical_group is not UNSET:
            field_dict["hierarchicalGroup"] = hierarchical_group
        if career_scale is not UNSET:
            field_dict["careerScale"] = career_scale
        if job_category is not UNSET:
            field_dict["jobCategory"] = job_category
        if job_group is not UNSET:
            field_dict["jobGroup"] = job_group
        if job_group_start is not UNSET:
            field_dict["jobGroupStart"] = job_group_start
        if career_grade is not UNSET:
            field_dict["careerGrade"] = career_grade
        if career_grade_start is not UNSET:
            field_dict["careerGradeStart"] = career_grade_start
        if tg_grade is not UNSET:
            field_dict["tgGrade"] = tg_grade
        if is_hosted is not UNSET:
            field_dict["isHosted"] = is_hosted
        if is_teacher_or_reasearcher is not UNSET:
            field_dict["isTeacherOrReasearcher"] = is_teacher_or_reasearcher
        if is_staff is not UNSET:
            field_dict["isStaff"] = is_staff
        if is_teacher is not UNSET:
            field_dict["isTeacher"] = is_teacher
        if is_researcher is not UNSET:
            field_dict["isResearcher"] = is_researcher
        if is_secondary_school_teacher is not UNSET:
            field_dict["isSecondarySchoolTeacher"] = is_secondary_school_teacher
        if is_active is not UNSET:
            field_dict["isActive"] = is_active
        if categories is not UNSET:
            field_dict["categories"] = categories
        if is_pending is not UNSET:
            field_dict["isPending"] = is_pending
        if is_expired is not UNSET:
            field_dict["isExpired"] = is_expired

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.label import Label

        if not src_dict:
            return None
        d = src_dict.copy()
        matricule = d.pop("matricule", UNSET)

        _start_effective = d.pop("startEffective", UNSET)
        start_effective: Union[Unset, datetime.datetime]
        if isinstance(_start_effective, Unset):
            start_effective = UNSET
        else:
            start_effective = isoparse(_start_effective)

        _end_effective = d.pop("endEffective", UNSET)
        end_effective: Union[Unset, datetime.datetime]
        if isinstance(_end_effective, Unset):
            end_effective = UNSET
        else:
            end_effective = isoparse(_end_effective)

        decree = d.pop("decree", UNSET)

        order = d.pop("order", UNSET)

        line = d.pop("line", UNSET)

        type = d.pop("type", UNSET)

        validity = d.pop("validity", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, Label]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = Label.from_dict(_status)

        _hierarchical_group = d.pop("hierarchicalGroup", UNSET)
        hierarchical_group: Union[Unset, Label]
        if isinstance(_hierarchical_group, Unset):
            hierarchical_group = UNSET
        else:
            hierarchical_group = Label.from_dict(_hierarchical_group)

        _career_scale = d.pop("careerScale", UNSET)
        career_scale: Union[Unset, Label]
        if isinstance(_career_scale, Unset):
            career_scale = UNSET
        else:
            career_scale = Label.from_dict(_career_scale)

        _job_category = d.pop("jobCategory", UNSET)
        job_category: Union[Unset, Label]
        if isinstance(_job_category, Unset):
            job_category = UNSET
        else:
            job_category = Label.from_dict(_job_category)

        _job_group = d.pop("jobGroup", UNSET)
        job_group: Union[Unset, Label]
        if isinstance(_job_group, Unset):
            job_group = UNSET
        else:
            job_group = Label.from_dict(_job_group)

        _job_group_start = d.pop("jobGroupStart", UNSET)
        job_group_start: Union[Unset, datetime.datetime]
        if isinstance(_job_group_start, Unset):
            job_group_start = UNSET
        else:
            job_group_start = isoparse(_job_group_start)

        _career_grade = d.pop("careerGrade", UNSET)
        career_grade: Union[Unset, Label]
        if isinstance(_career_grade, Unset):
            career_grade = UNSET
        else:
            career_grade = Label.from_dict(_career_grade)

        _career_grade_start = d.pop("careerGradeStart", UNSET)
        career_grade_start: Union[Unset, datetime.datetime]
        if isinstance(_career_grade_start, Unset):
            career_grade_start = UNSET
        else:
            career_grade_start = isoparse(_career_grade_start)

        _tg_grade = d.pop("tgGrade", UNSET)
        tg_grade: Union[Unset, Label]
        if isinstance(_tg_grade, Unset):
            tg_grade = UNSET
        else:
            tg_grade = Label.from_dict(_tg_grade)

        is_hosted = d.pop("isHosted", UNSET)

        is_teacher_or_reasearcher = d.pop("isTeacherOrReasearcher", UNSET)

        is_staff = d.pop("isStaff", UNSET)

        is_teacher = d.pop("isTeacher", UNSET)

        is_researcher = d.pop("isResearcher", UNSET)

        is_secondary_school_teacher = d.pop("isSecondarySchoolTeacher", UNSET)

        is_active = d.pop("isActive", UNSET)

        categories = cast(List[str], d.pop("categories", UNSET))

        is_pending = d.pop("isPending", UNSET)

        is_expired = d.pop("isExpired", UNSET)

        career = cls(
            matricule=matricule,
            start_effective=start_effective,
            end_effective=end_effective,
            decree=decree,
            order=order,
            line=line,
            type=type,
            validity=validity,
            status=status,
            hierarchical_group=hierarchical_group,
            career_scale=career_scale,
            job_category=job_category,
            job_group=job_group,
            job_group_start=job_group_start,
            career_grade=career_grade,
            career_grade_start=career_grade_start,
            tg_grade=tg_grade,
            is_hosted=is_hosted,
            is_teacher_or_reasearcher=is_teacher_or_reasearcher,
            is_staff=is_staff,
            is_teacher=is_teacher,
            is_researcher=is_researcher,
            is_secondary_school_teacher=is_secondary_school_teacher,
            is_active=is_active,
            categories=categories,
            is_pending=is_pending,
            is_expired=is_expired,
        )

        career.additional_properties = d
        return career

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
