import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.job_type import JobType
    from ..models.label import Label
    from ..models.organizational_unit_lite import OrganizationalUnitLite


T = TypeVar("T", bound="ComplementaryActivity")


@_attrs_define
class ComplementaryActivity:
    """Activité supplémentaire d'un agent

    Attributes:
        matricule (Union[Unset, str]): Numéro de dossier
        start (Union[Unset, datetime.datetime]): Date de début effectif
        end (Union[Unset, datetime.datetime]): Date de fin effective
        organizational_unit (Union[Unset, OrganizationalUnitLite]): Unité Organisationnelle (UO)
        type (Union[Unset, Label]): Etiquette
        population (Union[Unset, Label]): Etiquette
        job_type (Union[Unset, JobType]): Emploi type
        service_mode (Union[Unset, Label]): Etiquette
        quota (Union[Unset, int]): Quotité (en pourcentage)
        is_active (Union[Unset, bool]): Témoin d'activité
        is_pending (Union[Unset, bool]): Témoin de mise en attente (pas encore commencée)
        is_expired (Union[Unset, bool]): Témoin d'expiration
    """

    matricule: Union[Unset, str] = UNSET
    start: Union[Unset, datetime.datetime] = UNSET
    end: Union[Unset, datetime.datetime] = UNSET
    organizational_unit: Union[Unset, "OrganizationalUnitLite"] = UNSET
    type: Union[Unset, "Label"] = UNSET
    population: Union[Unset, "Label"] = UNSET
    job_type: Union[Unset, "JobType"] = UNSET
    service_mode: Union[Unset, "Label"] = UNSET
    quota: Union[Unset, int] = UNSET
    is_active: Union[Unset, bool] = UNSET
    is_pending: Union[Unset, bool] = UNSET
    is_expired: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        matricule = self.matricule
        start: Union[Unset, str] = UNSET
        if not isinstance(self.start, Unset):
            start = self.start.isoformat()

        end: Union[Unset, str] = UNSET
        if not isinstance(self.end, Unset):
            end = self.end.isoformat()

        organizational_unit: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.organizational_unit, Unset):
            organizational_unit = self.organizational_unit.to_dict()

        type: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.to_dict()

        population: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.population, Unset):
            population = self.population.to_dict()

        job_type: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.job_type, Unset):
            job_type = self.job_type.to_dict()

        service_mode: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.service_mode, Unset):
            service_mode = self.service_mode.to_dict()

        quota = self.quota
        is_active = self.is_active
        is_pending = self.is_pending
        is_expired = self.is_expired

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if matricule is not UNSET:
            field_dict["matricule"] = matricule
        if start is not UNSET:
            field_dict["start"] = start
        if end is not UNSET:
            field_dict["end"] = end
        if organizational_unit is not UNSET:
            field_dict["organizationalUnit"] = organizational_unit
        if type is not UNSET:
            field_dict["type"] = type
        if population is not UNSET:
            field_dict["population"] = population
        if job_type is not UNSET:
            field_dict["jobType"] = job_type
        if service_mode is not UNSET:
            field_dict["serviceMode"] = service_mode
        if quota is not UNSET:
            field_dict["quota"] = quota
        if is_active is not UNSET:
            field_dict["isActive"] = is_active
        if is_pending is not UNSET:
            field_dict["isPending"] = is_pending
        if is_expired is not UNSET:
            field_dict["isExpired"] = is_expired

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.job_type import JobType
        from ..models.label import Label
        from ..models.organizational_unit_lite import OrganizationalUnitLite

        if not src_dict:
            return None
        d = src_dict.copy()
        matricule = d.pop("matricule", UNSET)

        _start = d.pop("start", UNSET)
        start: Union[Unset, datetime.datetime]
        if isinstance(_start, Unset):
            start = UNSET
        else:
            start = isoparse(_start)

        _end = d.pop("end", UNSET)
        end: Union[Unset, datetime.datetime]
        if isinstance(_end, Unset):
            end = UNSET
        else:
            end = isoparse(_end)

        _organizational_unit = d.pop("organizationalUnit", UNSET)
        organizational_unit: Union[Unset, OrganizationalUnitLite]
        if isinstance(_organizational_unit, Unset):
            organizational_unit = UNSET
        else:
            organizational_unit = OrganizationalUnitLite.from_dict(_organizational_unit)

        _type = d.pop("type", UNSET)
        type: Union[Unset, Label]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = Label.from_dict(_type)

        _population = d.pop("population", UNSET)
        population: Union[Unset, Label]
        if isinstance(_population, Unset):
            population = UNSET
        else:
            population = Label.from_dict(_population)

        _job_type = d.pop("jobType", UNSET)
        job_type: Union[Unset, JobType]
        if isinstance(_job_type, Unset):
            job_type = UNSET
        else:
            job_type = JobType.from_dict(_job_type)

        _service_mode = d.pop("serviceMode", UNSET)
        service_mode: Union[Unset, Label]
        if isinstance(_service_mode, Unset):
            service_mode = UNSET
        else:
            service_mode = Label.from_dict(_service_mode)

        quota = d.pop("quota", UNSET)

        is_active = d.pop("isActive", UNSET)

        is_pending = d.pop("isPending", UNSET)

        is_expired = d.pop("isExpired", UNSET)

        complementary_activity = cls(
            matricule=matricule,
            start=start,
            end=end,
            organizational_unit=organizational_unit,
            type=type,
            population=population,
            job_type=job_type,
            service_mode=service_mode,
            quota=quota,
            is_active=is_active,
            is_pending=is_pending,
            is_expired=is_expired,
        )

        complementary_activity.additional_properties = d
        return complementary_activity

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
