import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="ConnectorStatus")


@_attrs_define
class ConnectorStatus:
    """Informations un statut d'un connecteur vers une source de données distante

    Attributes:
        name (Union[Unset, str]): Nom du connecteur
        label (Union[Unset, str]): Description du connecteur
        type (Union[Unset, str]): Type de connexion
        target (Union[Unset, str]): Cible
        time (Union[Unset, datetime.datetime]): Date du relevé d'état
        is_active (Union[Unset, bool]): Témoin d'activité
    """

    name: Union[Unset, str] = UNSET
    label: Union[Unset, str] = UNSET
    type: Union[Unset, str] = UNSET
    target: Union[Unset, str] = UNSET
    time: Union[Unset, datetime.datetime] = UNSET
    is_active: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        label = self.label
        type = self.type
        target = self.target
        time: Union[Unset, str] = UNSET
        if not isinstance(self.time, Unset):
            time = self.time.isoformat()

        is_active = self.is_active

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if label is not UNSET:
            field_dict["label"] = label
        if type is not UNSET:
            field_dict["type"] = type
        if target is not UNSET:
            field_dict["target"] = target
        if time is not UNSET:
            field_dict["time"] = time
        if is_active is not UNSET:
            field_dict["isActive"] = is_active

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        if not src_dict:
            return None
        d = src_dict.copy()
        name = d.pop("name", UNSET)

        label = d.pop("label", UNSET)

        type = d.pop("type", UNSET)

        target = d.pop("target", UNSET)

        _time = d.pop("time", UNSET)
        time: Union[Unset, datetime.datetime]
        if isinstance(_time, Unset):
            time = UNSET
        else:
            time = isoparse(_time)

        is_active = d.pop("isActive", UNSET)

        connector_status = cls(
            name=name,
            label=label,
            type=type,
            target=target,
            time=time,
            is_active=is_active,
        )

        connector_status.additional_properties = d
        return connector_status

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
