import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.label import Label


T = TypeVar("T", bound="Contract")


@_attrs_define
class Contract:
    """Contrat d'un agent

    Attributes:
        matricule (Union[Unset, str]): Numéro de dossier
        rank (Union[Unset, int]): Rang
        start_effective (Union[Unset, datetime.datetime]): Date de début
        end_effective (Union[Unset, datetime.datetime]): Date de fin effective
        end_planned (Union[Unset, datetime.datetime]): Date de fin prévue
        signature (Union[Unset, datetime.datetime]): Date de signature
        decree (Union[Unset, str]): Numéro d'arrêté/de décision
        amendment_reference (Union[Unset, str]): Numéro d'avenant
        compensation_index (Union[Unset, int]): Indice de rémunération
        comment (Union[Unset, str]): Commentaire
        validity (Union[Unset, int]): Témoin de validité des informaitons (4 = valide)
        job_category (Union[Unset, Label]): Etiquette
        contract_kind (Union[Unset, Label]): Etiquette
        contract_type (Union[Unset, Label]): Etiquette
        tg_grade (Union[Unset, Label]): Etiquette
        extension_reason (Union[Unset, Label]): Etiquette
        is_active (Union[Unset, bool]): Témoin d'activité
        is_pending (Union[Unset, bool]): Témoin de mise en attente (pas encore commencé)
        is_expired (Union[Unset, bool]): Témoin d'expiration
    """

    matricule: Union[Unset, str] = UNSET
    rank: Union[Unset, int] = UNSET
    start_effective: Union[Unset, datetime.datetime] = UNSET
    end_effective: Union[Unset, datetime.datetime] = UNSET
    end_planned: Union[Unset, datetime.datetime] = UNSET
    signature: Union[Unset, datetime.datetime] = UNSET
    decree: Union[Unset, str] = UNSET
    amendment_reference: Union[Unset, str] = UNSET
    compensation_index: Union[Unset, int] = UNSET
    comment: Union[Unset, str] = UNSET
    validity: Union[Unset, int] = UNSET
    job_category: Union[Unset, "Label"] = UNSET
    contract_kind: Union[Unset, "Label"] = UNSET
    contract_type: Union[Unset, "Label"] = UNSET
    tg_grade: Union[Unset, "Label"] = UNSET
    extension_reason: Union[Unset, "Label"] = UNSET
    is_active: Union[Unset, bool] = UNSET
    is_pending: Union[Unset, bool] = UNSET
    is_expired: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        matricule = self.matricule
        rank = self.rank
        start_effective: Union[Unset, str] = UNSET
        if not isinstance(self.start_effective, Unset):
            start_effective = self.start_effective.isoformat()

        end_effective: Union[Unset, str] = UNSET
        if not isinstance(self.end_effective, Unset):
            end_effective = self.end_effective.isoformat()

        end_planned: Union[Unset, str] = UNSET
        if not isinstance(self.end_planned, Unset):
            end_planned = self.end_planned.isoformat()

        signature: Union[Unset, str] = UNSET
        if not isinstance(self.signature, Unset):
            signature = self.signature.isoformat()

        decree = self.decree
        amendment_reference = self.amendment_reference
        compensation_index = self.compensation_index
        comment = self.comment
        validity = self.validity
        job_category: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.job_category, Unset):
            job_category = self.job_category.to_dict()

        contract_kind: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.contract_kind, Unset):
            contract_kind = self.contract_kind.to_dict()

        contract_type: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.contract_type, Unset):
            contract_type = self.contract_type.to_dict()

        tg_grade: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.tg_grade, Unset):
            tg_grade = self.tg_grade.to_dict()

        extension_reason: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.extension_reason, Unset):
            extension_reason = self.extension_reason.to_dict()

        is_active = self.is_active
        is_pending = self.is_pending
        is_expired = self.is_expired

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if matricule is not UNSET:
            field_dict["matricule"] = matricule
        if rank is not UNSET:
            field_dict["rank"] = rank
        if start_effective is not UNSET:
            field_dict["startEffective"] = start_effective
        if end_effective is not UNSET:
            field_dict["endEffective"] = end_effective
        if end_planned is not UNSET:
            field_dict["endPlanned"] = end_planned
        if signature is not UNSET:
            field_dict["signature"] = signature
        if decree is not UNSET:
            field_dict["decree"] = decree
        if amendment_reference is not UNSET:
            field_dict["amendmentReference"] = amendment_reference
        if compensation_index is not UNSET:
            field_dict["compensationIndex"] = compensation_index
        if comment is not UNSET:
            field_dict["comment"] = comment
        if validity is not UNSET:
            field_dict["validity"] = validity
        if job_category is not UNSET:
            field_dict["jobCategory"] = job_category
        if contract_kind is not UNSET:
            field_dict["contractKind"] = contract_kind
        if contract_type is not UNSET:
            field_dict["contractType"] = contract_type
        if tg_grade is not UNSET:
            field_dict["tgGrade"] = tg_grade
        if extension_reason is not UNSET:
            field_dict["extensionReason"] = extension_reason
        if is_active is not UNSET:
            field_dict["isActive"] = is_active
        if is_pending is not UNSET:
            field_dict["isPending"] = is_pending
        if is_expired is not UNSET:
            field_dict["isExpired"] = is_expired

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.label import Label

        if not src_dict:
            return None
        d = src_dict.copy()
        matricule = d.pop("matricule", UNSET)

        rank = d.pop("rank", UNSET)

        _start_effective = d.pop("startEffective", UNSET)
        start_effective: Union[Unset, datetime.datetime]
        if isinstance(_start_effective, Unset):
            start_effective = UNSET
        else:
            start_effective = isoparse(_start_effective)

        _end_effective = d.pop("endEffective", UNSET)
        end_effective: Union[Unset, datetime.datetime]
        if isinstance(_end_effective, Unset):
            end_effective = UNSET
        else:
            end_effective = isoparse(_end_effective)

        _end_planned = d.pop("endPlanned", UNSET)
        end_planned: Union[Unset, datetime.datetime]
        if isinstance(_end_planned, Unset):
            end_planned = UNSET
        else:
            end_planned = isoparse(_end_planned)

        _signature = d.pop("signature", UNSET)
        signature: Union[Unset, datetime.datetime]
        if isinstance(_signature, Unset):
            signature = UNSET
        else:
            signature = isoparse(_signature)

        decree = d.pop("decree", UNSET)

        amendment_reference = d.pop("amendmentReference", UNSET)

        compensation_index = d.pop("compensationIndex", UNSET)

        comment = d.pop("comment", UNSET)

        validity = d.pop("validity", UNSET)

        _job_category = d.pop("jobCategory", UNSET)
        job_category: Union[Unset, Label]
        if isinstance(_job_category, Unset):
            job_category = UNSET
        else:
            job_category = Label.from_dict(_job_category)

        _contract_kind = d.pop("contractKind", UNSET)
        contract_kind: Union[Unset, Label]
        if isinstance(_contract_kind, Unset):
            contract_kind = UNSET
        else:
            contract_kind = Label.from_dict(_contract_kind)

        _contract_type = d.pop("contractType", UNSET)
        contract_type: Union[Unset, Label]
        if isinstance(_contract_type, Unset):
            contract_type = UNSET
        else:
            contract_type = Label.from_dict(_contract_type)

        _tg_grade = d.pop("tgGrade", UNSET)
        tg_grade: Union[Unset, Label]
        if isinstance(_tg_grade, Unset):
            tg_grade = UNSET
        else:
            tg_grade = Label.from_dict(_tg_grade)

        _extension_reason = d.pop("extensionReason", UNSET)
        extension_reason: Union[Unset, Label]
        if isinstance(_extension_reason, Unset):
            extension_reason = UNSET
        else:
            extension_reason = Label.from_dict(_extension_reason)

        is_active = d.pop("isActive", UNSET)

        is_pending = d.pop("isPending", UNSET)

        is_expired = d.pop("isExpired", UNSET)

        contract = cls(
            matricule=matricule,
            rank=rank,
            start_effective=start_effective,
            end_effective=end_effective,
            end_planned=end_planned,
            signature=signature,
            decree=decree,
            amendment_reference=amendment_reference,
            compensation_index=compensation_index,
            comment=comment,
            validity=validity,
            job_category=job_category,
            contract_kind=contract_kind,
            contract_type=contract_type,
            tg_grade=tg_grade,
            extension_reason=extension_reason,
            is_active=is_active,
            is_pending=is_pending,
            is_expired=is_expired,
        )

        contract.additional_properties = d
        return contract

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
