import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.organizational_unit_lite import OrganizationalUnitLite


T = TypeVar("T", bound="FailedEnd")


@_attrs_define
class FailedEnd:
    """Dossier potentiellement en erreur par rapport aux dates de fin (contrats et/ou situations administratives)

    Attributes:
        matricule (Union[Unset, str]): Numéro de dossier
        organizational_unit (Union[Unset, OrganizationalUnitLite]): Unité Organisationnelle (UO)
        contract_end (Union[Unset, datetime.datetime]): Date de fin de contrat
        situation_end (Union[Unset, datetime.datetime]): Date de fin de la situation administrative
    """

    matricule: Union[Unset, str] = UNSET
    organizational_unit: Union[Unset, "OrganizationalUnitLite"] = UNSET
    contract_end: Union[Unset, datetime.datetime] = UNSET
    situation_end: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        matricule = self.matricule
        organizational_unit: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.organizational_unit, Unset):
            organizational_unit = self.organizational_unit.to_dict()

        contract_end: Union[Unset, str] = UNSET
        if not isinstance(self.contract_end, Unset):
            contract_end = self.contract_end.isoformat()

        situation_end: Union[Unset, str] = UNSET
        if not isinstance(self.situation_end, Unset):
            situation_end = self.situation_end.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if matricule is not UNSET:
            field_dict["matricule"] = matricule
        if organizational_unit is not UNSET:
            field_dict["organizationalUnit"] = organizational_unit
        if contract_end is not UNSET:
            field_dict["contractEnd"] = contract_end
        if situation_end is not UNSET:
            field_dict["situationEnd"] = situation_end

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.organizational_unit_lite import OrganizationalUnitLite

        if not src_dict:
            return None
        d = src_dict.copy()
        matricule = d.pop("matricule", UNSET)

        _organizational_unit = d.pop("organizationalUnit", UNSET)
        organizational_unit: Union[Unset, OrganizationalUnitLite]
        if isinstance(_organizational_unit, Unset):
            organizational_unit = UNSET
        else:
            organizational_unit = OrganizationalUnitLite.from_dict(_organizational_unit)

        _contract_end = d.pop("contractEnd", UNSET)
        contract_end: Union[Unset, datetime.datetime]
        if isinstance(_contract_end, Unset):
            contract_end = UNSET
        else:
            contract_end = isoparse(_contract_end)

        _situation_end = d.pop("situationEnd", UNSET)
        situation_end: Union[Unset, datetime.datetime]
        if isinstance(_situation_end, Unset):
            situation_end = UNSET
        else:
            situation_end = isoparse(_situation_end)

        failed_end = cls(
            matricule=matricule,
            organizational_unit=organizational_unit,
            contract_end=contract_end,
            situation_end=situation_end,
        )

        failed_end.additional_properties = d
        return failed_end

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
