import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.organizational_unit_lite import OrganizationalUnitLite


T = TypeVar("T", bound="FunctionalAssignment")


@_attrs_define
class FunctionalAssignment:
    """Affectation fonctionnelle d'un agent

    Attributes:
        matricule (Union[Unset, str]): Numéro de dossier
        start_effective (Union[Unset, datetime.datetime]): Date de début effectif
        end_effective (Union[Unset, datetime.datetime]): Date de fin effective
        organizational_unit (Union[Unset, OrganizationalUnitLite]): Unité Organisationnelle (UO)
        quota (Union[Unset, int]): Quotité (en pourcentage)
        is_active (Union[Unset, bool]): Témoin d'activité
        is_pending (Union[Unset, bool]): Témoin de mise en attente (pas encore commencée)
        is_expired (Union[Unset, bool]): Témoin d'expiration
    """

    matricule: Union[Unset, str] = UNSET
    start_effective: Union[Unset, datetime.datetime] = UNSET
    end_effective: Union[Unset, datetime.datetime] = UNSET
    organizational_unit: Union[Unset, "OrganizationalUnitLite"] = UNSET
    quota: Union[Unset, int] = UNSET
    is_active: Union[Unset, bool] = UNSET
    is_pending: Union[Unset, bool] = UNSET
    is_expired: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        matricule = self.matricule
        start_effective: Union[Unset, str] = UNSET
        if not isinstance(self.start_effective, Unset):
            start_effective = self.start_effective.isoformat()

        end_effective: Union[Unset, str] = UNSET
        if not isinstance(self.end_effective, Unset):
            end_effective = self.end_effective.isoformat()

        organizational_unit: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.organizational_unit, Unset):
            organizational_unit = self.organizational_unit.to_dict()

        quota = self.quota
        is_active = self.is_active
        is_pending = self.is_pending
        is_expired = self.is_expired

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if matricule is not UNSET:
            field_dict["matricule"] = matricule
        if start_effective is not UNSET:
            field_dict["startEffective"] = start_effective
        if end_effective is not UNSET:
            field_dict["endEffective"] = end_effective
        if organizational_unit is not UNSET:
            field_dict["organizationalUnit"] = organizational_unit
        if quota is not UNSET:
            field_dict["quota"] = quota
        if is_active is not UNSET:
            field_dict["isActive"] = is_active
        if is_pending is not UNSET:
            field_dict["isPending"] = is_pending
        if is_expired is not UNSET:
            field_dict["isExpired"] = is_expired

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.organizational_unit_lite import OrganizationalUnitLite

        if not src_dict:
            return None
        d = src_dict.copy()
        matricule = d.pop("matricule", UNSET)

        _start_effective = d.pop("startEffective", UNSET)
        start_effective: Union[Unset, datetime.datetime]
        if isinstance(_start_effective, Unset):
            start_effective = UNSET
        else:
            start_effective = isoparse(_start_effective)

        _end_effective = d.pop("endEffective", UNSET)
        end_effective: Union[Unset, datetime.datetime]
        if isinstance(_end_effective, Unset):
            end_effective = UNSET
        else:
            end_effective = isoparse(_end_effective)

        _organizational_unit = d.pop("organizationalUnit", UNSET)
        organizational_unit: Union[Unset, OrganizationalUnitLite]
        if isinstance(_organizational_unit, Unset):
            organizational_unit = UNSET
        else:
            organizational_unit = OrganizationalUnitLite.from_dict(_organizational_unit)

        quota = d.pop("quota", UNSET)

        is_active = d.pop("isActive", UNSET)

        is_pending = d.pop("isPending", UNSET)

        is_expired = d.pop("isExpired", UNSET)

        functional_assignment = cls(
            matricule=matricule,
            start_effective=start_effective,
            end_effective=end_effective,
            organizational_unit=organizational_unit,
            quota=quota,
            is_active=is_active,
            is_pending=is_pending,
            is_expired=is_expired,
        )

        functional_assignment.additional_properties = d
        return functional_assignment

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
