import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.organizational_unit_lite import OrganizationalUnitLite


T = TypeVar("T", bound="HierarchicalAssignmentPreview")


@_attrs_define
class HierarchicalAssignmentPreview:
    """Affectation principale (hiérarchique) d'un agent

    Attributes:
        matricule (Union[Unset, str]): Numéro de dossier
        start (Union[Unset, datetime.datetime]): Date de début
        end (Union[Unset, datetime.datetime]): Date de fin
        organizational_unit (Union[Unset, OrganizationalUnitLite]): Unité Organisationnelle (UO)
        active (Union[Unset, bool]):
    """

    matricule: Union[Unset, str] = UNSET
    start: Union[Unset, datetime.datetime] = UNSET
    end: Union[Unset, datetime.datetime] = UNSET
    organizational_unit: Union[Unset, "OrganizationalUnitLite"] = UNSET
    active: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        matricule = self.matricule
        start: Union[Unset, str] = UNSET
        if not isinstance(self.start, Unset):
            start = self.start.isoformat()

        end: Union[Unset, str] = UNSET
        if not isinstance(self.end, Unset):
            end = self.end.isoformat()

        organizational_unit: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.organizational_unit, Unset):
            organizational_unit = self.organizational_unit.to_dict()

        active = self.active

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if matricule is not UNSET:
            field_dict["matricule"] = matricule
        if start is not UNSET:
            field_dict["start"] = start
        if end is not UNSET:
            field_dict["end"] = end
        if organizational_unit is not UNSET:
            field_dict["organizationalUnit"] = organizational_unit
        if active is not UNSET:
            field_dict["active"] = active

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.organizational_unit_lite import OrganizationalUnitLite

        if not src_dict:
            return None
        d = src_dict.copy()
        matricule = d.pop("matricule", UNSET)

        _start = d.pop("start", UNSET)
        start: Union[Unset, datetime.datetime]
        if isinstance(_start, Unset):
            start = UNSET
        else:
            start = isoparse(_start)

        _end = d.pop("end", UNSET)
        end: Union[Unset, datetime.datetime]
        if isinstance(_end, Unset):
            end = UNSET
        else:
            end = isoparse(_end)

        _organizational_unit = d.pop("organizationalUnit", UNSET)
        organizational_unit: Union[Unset, OrganizationalUnitLite]
        if isinstance(_organizational_unit, Unset):
            organizational_unit = UNSET
        else:
            organizational_unit = OrganizationalUnitLite.from_dict(_organizational_unit)

        active = d.pop("active", UNSET)

        hierarchical_assignment_preview = cls(
            matricule=matricule,
            start=start,
            end=end,
            organizational_unit=organizational_unit,
            active=active,
        )

        hierarchical_assignment_preview.additional_properties = d
        return hierarchical_assignment_preview

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
