from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.label import Label


T = TypeVar("T", bound="JobType")


@_attrs_define
class JobType:
    """Emploi type

    Attributes:
        code (Union[Unset, str]): Code identificateur
        shortname (Union[Unset, str]): Libellé court
        longname (Union[Unset, str]): Libellé long
        is_active (Union[Unset, bool]): Témoin d'activité
        classification (Union[Unset, str]): Classification
        profesional_family (Union[Unset, Label]): Etiquette
    """

    code: Union[Unset, str] = UNSET
    shortname: Union[Unset, str] = UNSET
    longname: Union[Unset, str] = UNSET
    is_active: Union[Unset, bool] = UNSET
    classification: Union[Unset, str] = UNSET
    profesional_family: Union[Unset, "Label"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        code = self.code
        shortname = self.shortname
        longname = self.longname
        is_active = self.is_active
        classification = self.classification
        profesional_family: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profesional_family, Unset):
            profesional_family = self.profesional_family.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if code is not UNSET:
            field_dict["code"] = code
        if shortname is not UNSET:
            field_dict["shortname"] = shortname
        if longname is not UNSET:
            field_dict["longname"] = longname
        if is_active is not UNSET:
            field_dict["isActive"] = is_active
        if classification is not UNSET:
            field_dict["classification"] = classification
        if profesional_family is not UNSET:
            field_dict["profesionalFamily"] = profesional_family

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.label import Label

        if not src_dict:
            return None
        d = src_dict.copy()
        code = d.pop("code", UNSET)

        shortname = d.pop("shortname", UNSET)

        longname = d.pop("longname", UNSET)

        is_active = d.pop("isActive", UNSET)

        classification = d.pop("classification", UNSET)

        _profesional_family = d.pop("profesionalFamily", UNSET)
        profesional_family: Union[Unset, Label]
        if isinstance(_profesional_family, Unset):
            profesional_family = UNSET
        else:
            profesional_family = Label.from_dict(_profesional_family)

        job_type = cls(
            code=code,
            shortname=shortname,
            longname=longname,
            is_active=is_active,
            classification=classification,
            profesional_family=profesional_family,
        )

        job_type.additional_properties = d
        return job_type

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
