import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.account import Account
    from ..models.label import Label


T = TypeVar("T", bound="Manager")


@_attrs_define
class Manager:
    """Informations minimales sur le responsable d'un agent

    Attributes:
        matricule (Union[Unset, str]): Numéro de dossier SIHAM
        account (Union[Unset, Account]): Informations minimales sur le compte d'un agent
        type (Union[Unset, Label]): Etiquette
        start (Union[Unset, datetime.datetime]): Date de début
        end (Union[Unset, datetime.datetime]): Date de fin
    """

    matricule: Union[Unset, str] = UNSET
    account: Union[Unset, "Account"] = UNSET
    type: Union[Unset, "Label"] = UNSET
    start: Union[Unset, datetime.datetime] = UNSET
    end: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        matricule = self.matricule
        account: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.account, Unset):
            account = self.account.to_dict()

        type: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.to_dict()

        start: Union[Unset, str] = UNSET
        if not isinstance(self.start, Unset):
            start = self.start.isoformat()

        end: Union[Unset, str] = UNSET
        if not isinstance(self.end, Unset):
            end = self.end.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if matricule is not UNSET:
            field_dict["matricule"] = matricule
        if account is not UNSET:
            field_dict["account"] = account
        if type is not UNSET:
            field_dict["type"] = type
        if start is not UNSET:
            field_dict["start"] = start
        if end is not UNSET:
            field_dict["end"] = end

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.account import Account
        from ..models.label import Label

        if not src_dict:
            return None
        d = src_dict.copy()
        matricule = d.pop("matricule", UNSET)

        _account = d.pop("account", UNSET)
        account: Union[Unset, Account]
        if isinstance(_account, Unset):
            account = UNSET
        else:
            account = Account.from_dict(_account)

        _type = d.pop("type", UNSET)
        type: Union[Unset, Label]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = Label.from_dict(_type)

        _start = d.pop("start", UNSET)
        start: Union[Unset, datetime.datetime]
        if isinstance(_start, Unset):
            start = UNSET
        else:
            start = isoparse(_start)

        _end = d.pop("end", UNSET)
        end: Union[Unset, datetime.datetime]
        if isinstance(_end, Unset):
            end = UNSET
        else:
            end = isoparse(_end)

        manager = cls(
            matricule=matricule,
            account=account,
            type=type,
            start=start,
            end=end,
        )

        manager.additional_properties = d
        return manager

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
