import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="Modification")


@_attrs_define
class Modification:
    """Relevé de modification sur un dossier agent

    Attributes:
        modified (Union[Unset, datetime.datetime]): Date de modification
        matricule (Union[Unset, str]): Maticule du dossier manipulé
        modification_type_code (Union[Unset, str]): Type de modification
    """

    modified: Union[Unset, datetime.datetime] = UNSET
    matricule: Union[Unset, str] = UNSET
    modification_type_code: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        modified: Union[Unset, str] = UNSET
        if not isinstance(self.modified, Unset):
            modified = self.modified.isoformat()

        matricule = self.matricule
        modification_type_code = self.modification_type_code

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if modified is not UNSET:
            field_dict["modified"] = modified
        if matricule is not UNSET:
            field_dict["matricule"] = matricule
        if modification_type_code is not UNSET:
            field_dict["modificationTypeCode"] = modification_type_code

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        if not src_dict:
            return None
        d = src_dict.copy()
        _modified = d.pop("modified", UNSET)
        modified: Union[Unset, datetime.datetime]
        if isinstance(_modified, Unset):
            modified = UNSET
        else:
            modified = isoparse(_modified)

        matricule = d.pop("matricule", UNSET)

        modification_type_code = d.pop("modificationTypeCode", UNSET)

        modification = cls(
            modified=modified,
            matricule=matricule,
            modification_type_code=modification_type_code,
        )

        modification.additional_properties = d
        return modification

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
