import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="ModificationDetails")


@_attrs_define
class ModificationDetails:
    """Détails de modification sur un dossier agent

    Attributes:
        matricule (Union[Unset, str]): Maticule du dossier manipulé
        modified (Union[Unset, datetime.datetime]): Date de modification
        operator_matricule (Union[Unset, str]): Maticule du gestionnaure ayant effectuer l'opération
        operation_type_code (Union[Unset, str]): Type d'opération effectuée
        targeted_table (Union[Unset, str]): Table cible contenant l'information modifiée
    """

    matricule: Union[Unset, str] = UNSET
    modified: Union[Unset, datetime.datetime] = UNSET
    operator_matricule: Union[Unset, str] = UNSET
    operation_type_code: Union[Unset, str] = UNSET
    targeted_table: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        matricule = self.matricule
        modified: Union[Unset, str] = UNSET
        if not isinstance(self.modified, Unset):
            modified = self.modified.isoformat()

        operator_matricule = self.operator_matricule
        operation_type_code = self.operation_type_code
        targeted_table = self.targeted_table

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if matricule is not UNSET:
            field_dict["matricule"] = matricule
        if modified is not UNSET:
            field_dict["modified"] = modified
        if operator_matricule is not UNSET:
            field_dict["operatorMatricule"] = operator_matricule
        if operation_type_code is not UNSET:
            field_dict["operationTypeCode"] = operation_type_code
        if targeted_table is not UNSET:
            field_dict["targetedTable"] = targeted_table

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        if not src_dict:
            return None
        d = src_dict.copy()
        matricule = d.pop("matricule", UNSET)

        _modified = d.pop("modified", UNSET)
        modified: Union[Unset, datetime.datetime]
        if isinstance(_modified, Unset):
            modified = UNSET
        else:
            modified = isoparse(_modified)

        operator_matricule = d.pop("operatorMatricule", UNSET)

        operation_type_code = d.pop("operationTypeCode", UNSET)

        targeted_table = d.pop("targetedTable", UNSET)

        modification_details = cls(
            matricule=matricule,
            modified=modified,
            operator_matricule=operator_matricule,
            operation_type_code=operation_type_code,
            targeted_table=targeted_table,
        )

        modification_details.additional_properties = d
        return modification_details

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
