from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.label import Label
    from ..models.manager import Manager
    from ..models.ohris_agent_assignments import OhrisAgentAssignments
    from ..models.relative import Relative
    from ..models.service_mode import ServiceMode


T = TypeVar("T", bound="OhrisAgent")


@_attrs_define
class OhrisAgent:
    """
    Attributes:
        matricule (Union[Unset, str]): Numéro de dossier SIHAM
        civility (Union[Unset, Label]): Etiquette
        surname (Union[Unset, str]): Nom patronymique
        usualname (Union[Unset, str]): Nom usuel
        usual_firstname (Union[Unset, str]): Prénom usuel
        login (Union[Unset, str]): Identifiant de connexion
        mail (Union[Unset, str]): Adresse mail professionnelle
        manager (Union[Unset, Manager]): Informations minimales sur le responsable d'un agent
        statuses (Union[Unset, List['Label']]): Statut
        assignments (Union[Unset, OhrisAgentAssignments]): Collection de compilation d'affectations hierarchiques en
            vigueur
        service_modes (Union[Unset, List['ServiceMode']]): Collection d'informations des modalités de services actives
        children (Union[Unset, List['Relative']]): Collection d'informations minimales sur les enfants
    """

    matricule: Union[Unset, str] = UNSET
    civility: Union[Unset, "Label"] = UNSET
    surname: Union[Unset, str] = UNSET
    usualname: Union[Unset, str] = UNSET
    usual_firstname: Union[Unset, str] = UNSET
    login: Union[Unset, str] = UNSET
    mail: Union[Unset, str] = UNSET
    manager: Union[Unset, "Manager"] = UNSET
    statuses: Union[Unset, List["Label"]] = UNSET
    assignments: Union[Unset, "OhrisAgentAssignments"] = UNSET
    service_modes: Union[Unset, List["ServiceMode"]] = UNSET
    children: Union[Unset, List["Relative"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        matricule = self.matricule
        civility: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.civility, Unset):
            civility = self.civility.to_dict()

        surname = self.surname
        usualname = self.usualname
        usual_firstname = self.usual_firstname
        login = self.login
        mail = self.mail
        manager: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.manager, Unset):
            manager = self.manager.to_dict()

        statuses: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.statuses, Unset):
            statuses = []
            for statuses_item_data in self.statuses:
                statuses_item = statuses_item_data.to_dict()

                statuses.append(statuses_item)

        assignments: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.assignments, Unset):
            assignments = self.assignments.to_dict()

        service_modes: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.service_modes, Unset):
            service_modes = []
            for service_modes_item_data in self.service_modes:
                service_modes_item = service_modes_item_data.to_dict()

                service_modes.append(service_modes_item)

        children: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.children, Unset):
            children = []
            for children_item_data in self.children:
                children_item = children_item_data.to_dict()

                children.append(children_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if matricule is not UNSET:
            field_dict["matricule"] = matricule
        if civility is not UNSET:
            field_dict["civility"] = civility
        if surname is not UNSET:
            field_dict["surname"] = surname
        if usualname is not UNSET:
            field_dict["usualname"] = usualname
        if usual_firstname is not UNSET:
            field_dict["usualFirstname"] = usual_firstname
        if login is not UNSET:
            field_dict["login"] = login
        if mail is not UNSET:
            field_dict["mail"] = mail
        if manager is not UNSET:
            field_dict["manager"] = manager
        if statuses is not UNSET:
            field_dict["statuses"] = statuses
        if assignments is not UNSET:
            field_dict["assignments"] = assignments
        if service_modes is not UNSET:
            field_dict["serviceModes"] = service_modes
        if children is not UNSET:
            field_dict["children"] = children

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.label import Label
        from ..models.manager import Manager
        from ..models.ohris_agent_assignments import OhrisAgentAssignments
        from ..models.relative import Relative
        from ..models.service_mode import ServiceMode

        if not src_dict:
            return None
        d = src_dict.copy()
        matricule = d.pop("matricule", UNSET)

        _civility = d.pop("civility", UNSET)
        civility: Union[Unset, Label]
        if isinstance(_civility, Unset):
            civility = UNSET
        else:
            civility = Label.from_dict(_civility)

        surname = d.pop("surname", UNSET)

        usualname = d.pop("usualname", UNSET)

        usual_firstname = d.pop("usualFirstname", UNSET)

        login = d.pop("login", UNSET)

        mail = d.pop("mail", UNSET)

        _manager = d.pop("manager", UNSET)
        manager: Union[Unset, Manager]
        if isinstance(_manager, Unset):
            manager = UNSET
        else:
            manager = Manager.from_dict(_manager)

        statuses = []
        _statuses = d.pop("statuses", UNSET)
        for statuses_item_data in _statuses or []:
            statuses_item = Label.from_dict(statuses_item_data)

            statuses.append(statuses_item)

        _assignments = d.pop("assignments", UNSET)
        assignments: Union[Unset, OhrisAgentAssignments]
        if isinstance(_assignments, Unset):
            assignments = UNSET
        else:
            assignments = OhrisAgentAssignments.from_dict(_assignments)

        service_modes = []
        _service_modes = d.pop("serviceModes", UNSET)
        for service_modes_item_data in _service_modes or []:
            service_modes_item = ServiceMode.from_dict(service_modes_item_data)

            service_modes.append(service_modes_item)

        children = []
        _children = d.pop("children", UNSET)
        for children_item_data in _children or []:
            children_item = Relative.from_dict(children_item_data)

            children.append(children_item)

        ohris_agent = cls(
            matricule=matricule,
            civility=civility,
            surname=surname,
            usualname=usualname,
            usual_firstname=usual_firstname,
            login=login,
            mail=mail,
            manager=manager,
            statuses=statuses,
            assignments=assignments,
            service_modes=service_modes,
            children=children,
        )

        ohris_agent.additional_properties = d
        return ohris_agent

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
