import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.label import Label


T = TypeVar("T", bound="OrganizationalUnit")


@_attrs_define
class OrganizationalUnit:
    """Unité Organisationnelle (UO)

    Attributes:
        code (Union[Unset, str]): Code identifiant de la structure
        shortname (Union[Unset, str]): Libellé court
        longname (Union[Unset, str]): Libellé long
        address (Union[Unset, str]): Adresse postale
        city (Union[Unset, str]): Ville
        start (Union[Unset, datetime.datetime]): Date de début
        start_effective (Union[Unset, datetime.datetime]): Date de début effectif
        end (Union[Unset, datetime.datetime]): Date de fin
        link (Union[Unset, Label]): Etiquette
        category (Union[Unset, Label]): Etiquette
        is_active (Union[Unset, bool]): Témoin d'activité
        is_pending (Union[Unset, bool]): Témoin de mise en attente (pas encore commencée)
        is_closed (Union[Unset, bool]): Témoin d'expiration
        hierarchical_parent_code (Union[Unset, str]): Code de l'UO hierarchique parente
        functional_parent_code (Union[Unset, str]): Code de l'UO fonctionnelle parente
    """

    code: Union[Unset, str] = UNSET
    shortname: Union[Unset, str] = UNSET
    longname: Union[Unset, str] = UNSET
    address: Union[Unset, str] = UNSET
    city: Union[Unset, str] = UNSET
    start: Union[Unset, datetime.datetime] = UNSET
    start_effective: Union[Unset, datetime.datetime] = UNSET
    end: Union[Unset, datetime.datetime] = UNSET
    link: Union[Unset, "Label"] = UNSET
    category: Union[Unset, "Label"] = UNSET
    is_active: Union[Unset, bool] = UNSET
    is_pending: Union[Unset, bool] = UNSET
    is_closed: Union[Unset, bool] = UNSET
    hierarchical_parent_code: Union[Unset, str] = UNSET
    functional_parent_code: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        code = self.code
        shortname = self.shortname
        longname = self.longname
        address = self.address
        city = self.city
        start: Union[Unset, str] = UNSET
        if not isinstance(self.start, Unset):
            start = self.start.isoformat()

        start_effective: Union[Unset, str] = UNSET
        if not isinstance(self.start_effective, Unset):
            start_effective = self.start_effective.isoformat()

        end: Union[Unset, str] = UNSET
        if not isinstance(self.end, Unset):
            end = self.end.isoformat()

        link: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.link, Unset):
            link = self.link.to_dict()

        category: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.category, Unset):
            category = self.category.to_dict()

        is_active = self.is_active
        is_pending = self.is_pending
        is_closed = self.is_closed
        hierarchical_parent_code = self.hierarchical_parent_code
        functional_parent_code = self.functional_parent_code

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if code is not UNSET:
            field_dict["code"] = code
        if shortname is not UNSET:
            field_dict["shortname"] = shortname
        if longname is not UNSET:
            field_dict["longname"] = longname
        if address is not UNSET:
            field_dict["address"] = address
        if city is not UNSET:
            field_dict["city"] = city
        if start is not UNSET:
            field_dict["start"] = start
        if start_effective is not UNSET:
            field_dict["startEffective"] = start_effective
        if end is not UNSET:
            field_dict["end"] = end
        if link is not UNSET:
            field_dict["link"] = link
        if category is not UNSET:
            field_dict["category"] = category
        if is_active is not UNSET:
            field_dict["isActive"] = is_active
        if is_pending is not UNSET:
            field_dict["isPending"] = is_pending
        if is_closed is not UNSET:
            field_dict["isClosed"] = is_closed
        if hierarchical_parent_code is not UNSET:
            field_dict["hierarchicalParentCode"] = hierarchical_parent_code
        if functional_parent_code is not UNSET:
            field_dict["functionalParentCode"] = functional_parent_code

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.label import Label

        if not src_dict:
            return None
        d = src_dict.copy()
        code = d.pop("code", UNSET)

        shortname = d.pop("shortname", UNSET)

        longname = d.pop("longname", UNSET)

        address = d.pop("address", UNSET)

        city = d.pop("city", UNSET)

        _start = d.pop("start", UNSET)
        start: Union[Unset, datetime.datetime]
        if isinstance(_start, Unset):
            start = UNSET
        else:
            start = isoparse(_start)

        _start_effective = d.pop("startEffective", UNSET)
        start_effective: Union[Unset, datetime.datetime]
        if isinstance(_start_effective, Unset):
            start_effective = UNSET
        else:
            start_effective = isoparse(_start_effective)

        _end = d.pop("end", UNSET)
        end: Union[Unset, datetime.datetime]
        if isinstance(_end, Unset):
            end = UNSET
        else:
            end = isoparse(_end)

        _link = d.pop("link", UNSET)
        link: Union[Unset, Label]
        if isinstance(_link, Unset):
            link = UNSET
        else:
            link = Label.from_dict(_link)

        _category = d.pop("category", UNSET)
        category: Union[Unset, Label]
        if isinstance(_category, Unset):
            category = UNSET
        else:
            category = Label.from_dict(_category)

        is_active = d.pop("isActive", UNSET)

        is_pending = d.pop("isPending", UNSET)

        is_closed = d.pop("isClosed", UNSET)

        hierarchical_parent_code = d.pop("hierarchicalParentCode", UNSET)

        functional_parent_code = d.pop("functionalParentCode", UNSET)

        organizational_unit = cls(
            code=code,
            shortname=shortname,
            longname=longname,
            address=address,
            city=city,
            start=start,
            start_effective=start_effective,
            end=end,
            link=link,
            category=category,
            is_active=is_active,
            is_pending=is_pending,
            is_closed=is_closed,
            hierarchical_parent_code=hierarchical_parent_code,
            functional_parent_code=functional_parent_code,
        )

        organizational_unit.additional_properties = d
        return organizational_unit

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
