from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.label import Label


T = TypeVar("T", bound="OrganizationalUnitLite")


@_attrs_define
class OrganizationalUnitLite:
    """Unité Organisationnelle (UO)

    Attributes:
        code (Union[Unset, str]): Code identifiant de la structure
        shortname (Union[Unset, str]): Libellé court
        longname (Union[Unset, str]): Libellé long
        link (Union[Unset, Label]): Etiquette
        category (Union[Unset, Label]): Etiquette
        is_active (Union[Unset, bool]): Témoin d'activité
    """

    code: Union[Unset, str] = UNSET
    shortname: Union[Unset, str] = UNSET
    longname: Union[Unset, str] = UNSET
    link: Union[Unset, "Label"] = UNSET
    category: Union[Unset, "Label"] = UNSET
    is_active: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        code = self.code
        shortname = self.shortname
        longname = self.longname
        link: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.link, Unset):
            link = self.link.to_dict()

        category: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.category, Unset):
            category = self.category.to_dict()

        is_active = self.is_active

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if code is not UNSET:
            field_dict["code"] = code
        if shortname is not UNSET:
            field_dict["shortname"] = shortname
        if longname is not UNSET:
            field_dict["longname"] = longname
        if link is not UNSET:
            field_dict["link"] = link
        if category is not UNSET:
            field_dict["category"] = category
        if is_active is not UNSET:
            field_dict["isActive"] = is_active

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.label import Label

        if not src_dict:
            return None
        d = src_dict.copy()
        code = d.pop("code", UNSET)

        shortname = d.pop("shortname", UNSET)

        longname = d.pop("longname", UNSET)

        _link = d.pop("link", UNSET)
        link: Union[Unset, Label]
        if isinstance(_link, Unset):
            link = UNSET
        else:
            link = Label.from_dict(_link)

        _category = d.pop("category", UNSET)
        category: Union[Unset, Label]
        if isinstance(_category, Unset):
            category = UNSET
        else:
            category = Label.from_dict(_category)

        is_active = d.pop("isActive", UNSET)

        organizational_unit_lite = cls(
            code=code,
            shortname=shortname,
            longname=longname,
            link=link,
            category=category,
            is_active=is_active,
        )

        organizational_unit_lite.additional_properties = d
        return organizational_unit_lite

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
