import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="Permission")


@_attrs_define
class Permission:
    """Habilitation d'un individu

    Attributes:
        matricule (Union[Unset, str]): Matricule
        start (Union[Unset, datetime.datetime]): Date de début
        end (Union[Unset, datetime.datetime]): Date de fin
        type (Union[Unset, str]): Type
        code (Union[Unset, str]): Code
        label (Union[Unset, str]): libellé
        role (Union[Unset, str]): Rôle
        is_active (Union[Unset, bool]): Témoin d'activité
        is_pending (Union[Unset, bool]): Témoin de mise en attente (pas encore commencé)
        is_expired (Union[Unset, bool]): Témoin d'expiration
    """

    matricule: Union[Unset, str] = UNSET
    start: Union[Unset, datetime.datetime] = UNSET
    end: Union[Unset, datetime.datetime] = UNSET
    type: Union[Unset, str] = UNSET
    code: Union[Unset, str] = UNSET
    label: Union[Unset, str] = UNSET
    role: Union[Unset, str] = UNSET
    is_active: Union[Unset, bool] = UNSET
    is_pending: Union[Unset, bool] = UNSET
    is_expired: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        matricule = self.matricule
        start: Union[Unset, str] = UNSET
        if not isinstance(self.start, Unset):
            start = self.start.isoformat()

        end: Union[Unset, str] = UNSET
        if not isinstance(self.end, Unset):
            end = self.end.isoformat()

        type = self.type
        code = self.code
        label = self.label
        role = self.role
        is_active = self.is_active
        is_pending = self.is_pending
        is_expired = self.is_expired

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if matricule is not UNSET:
            field_dict["matricule"] = matricule
        if start is not UNSET:
            field_dict["start"] = start
        if end is not UNSET:
            field_dict["end"] = end
        if type is not UNSET:
            field_dict["type"] = type
        if code is not UNSET:
            field_dict["code"] = code
        if label is not UNSET:
            field_dict["label"] = label
        if role is not UNSET:
            field_dict["role"] = role
        if is_active is not UNSET:
            field_dict["isActive"] = is_active
        if is_pending is not UNSET:
            field_dict["isPending"] = is_pending
        if is_expired is not UNSET:
            field_dict["isExpired"] = is_expired

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        if not src_dict:
            return None
        d = src_dict.copy()
        matricule = d.pop("matricule", UNSET)

        _start = d.pop("start", UNSET)
        start: Union[Unset, datetime.datetime]
        if isinstance(_start, Unset):
            start = UNSET
        else:
            start = isoparse(_start)

        _end = d.pop("end", UNSET)
        end: Union[Unset, datetime.datetime]
        if isinstance(_end, Unset):
            end = UNSET
        else:
            end = isoparse(_end)

        type = d.pop("type", UNSET)

        code = d.pop("code", UNSET)

        label = d.pop("label", UNSET)

        role = d.pop("role", UNSET)

        is_active = d.pop("isActive", UNSET)

        is_pending = d.pop("isPending", UNSET)

        is_expired = d.pop("isExpired", UNSET)

        permission = cls(
            matricule=matricule,
            start=start,
            end=end,
            type=type,
            code=code,
            label=label,
            role=role,
            is_active=is_active,
            is_pending=is_pending,
            is_expired=is_expired,
        )

        permission.additional_properties = d
        return permission

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
