import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.label import Label


T = TypeVar("T", bound="PersonalDetails")


@_attrs_define
class PersonalDetails:
    """Informations personnelles d'un agent

    Attributes:
        matricule (Union[Unset, str]): Numéro de dossier SIHAM
        civility (Union[Unset, Label]): Etiquette
        surname (Union[Unset, str]): Nom patronymique
        usualname (Union[Unset, str]): Nom usuel
        usual_firstname (Union[Unset, str]): Prénom usuel
        displayname (Union[Unset, str]): Nom d'affichage
        insee (Union[Unset, str]): Numéro INSEE
        birthday (Union[Unset, datetime.datetime]): Date de naissance
        firstname (Union[Unset, str]): Premier prénom
        middlename (Union[Unset, str]): Second prénom
        thirdname (Union[Unset, str]): Troisème prénom
        fourthname (Union[Unset, str]): Quatrième prénom
        forenames (Union[Unset, List[str]]): Liste de tous les prénoms
    """

    matricule: Union[Unset, str] = UNSET
    civility: Union[Unset, "Label"] = UNSET
    surname: Union[Unset, str] = UNSET
    usualname: Union[Unset, str] = UNSET
    usual_firstname: Union[Unset, str] = UNSET
    displayname: Union[Unset, str] = UNSET
    insee: Union[Unset, str] = UNSET
    birthday: Union[Unset, datetime.datetime] = UNSET
    firstname: Union[Unset, str] = UNSET
    middlename: Union[Unset, str] = UNSET
    thirdname: Union[Unset, str] = UNSET
    fourthname: Union[Unset, str] = UNSET
    forenames: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        matricule = self.matricule
        civility: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.civility, Unset):
            civility = self.civility.to_dict()

        surname = self.surname
        usualname = self.usualname
        usual_firstname = self.usual_firstname
        displayname = self.displayname
        insee = self.insee
        birthday: Union[Unset, str] = UNSET
        if not isinstance(self.birthday, Unset):
            birthday = self.birthday.isoformat()

        firstname = self.firstname
        middlename = self.middlename
        thirdname = self.thirdname
        fourthname = self.fourthname
        forenames: Union[Unset, List[str]] = UNSET
        if not isinstance(self.forenames, Unset):
            forenames = self.forenames

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if matricule is not UNSET:
            field_dict["matricule"] = matricule
        if civility is not UNSET:
            field_dict["civility"] = civility
        if surname is not UNSET:
            field_dict["surname"] = surname
        if usualname is not UNSET:
            field_dict["usualname"] = usualname
        if usual_firstname is not UNSET:
            field_dict["usualFirstname"] = usual_firstname
        if displayname is not UNSET:
            field_dict["displayname"] = displayname
        if insee is not UNSET:
            field_dict["insee"] = insee
        if birthday is not UNSET:
            field_dict["birthday"] = birthday
        if firstname is not UNSET:
            field_dict["firstname"] = firstname
        if middlename is not UNSET:
            field_dict["middlename"] = middlename
        if thirdname is not UNSET:
            field_dict["thirdname"] = thirdname
        if fourthname is not UNSET:
            field_dict["fourthname"] = fourthname
        if forenames is not UNSET:
            field_dict["forenames"] = forenames

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.label import Label

        if not src_dict:
            return None
        d = src_dict.copy()
        matricule = d.pop("matricule", UNSET)

        _civility = d.pop("civility", UNSET)
        civility: Union[Unset, Label]
        if isinstance(_civility, Unset):
            civility = UNSET
        else:
            civility = Label.from_dict(_civility)

        surname = d.pop("surname", UNSET)

        usualname = d.pop("usualname", UNSET)

        usual_firstname = d.pop("usualFirstname", UNSET)

        displayname = d.pop("displayname", UNSET)

        insee = d.pop("insee", UNSET)

        _birthday = d.pop("birthday", UNSET)
        birthday: Union[Unset, datetime.datetime]
        if isinstance(_birthday, Unset):
            birthday = UNSET
        else:
            birthday = isoparse(_birthday)

        firstname = d.pop("firstname", UNSET)

        middlename = d.pop("middlename", UNSET)

        thirdname = d.pop("thirdname", UNSET)

        fourthname = d.pop("fourthname", UNSET)

        forenames = cast(List[str], d.pop("forenames", UNSET))

        personal_details = cls(
            matricule=matricule,
            civility=civility,
            surname=surname,
            usualname=usualname,
            usual_firstname=usual_firstname,
            displayname=displayname,
            insee=insee,
            birthday=birthday,
            firstname=firstname,
            middlename=middlename,
            thirdname=thirdname,
            fourthname=fourthname,
            forenames=forenames,
        )

        personal_details.additional_properties = d
        return personal_details

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
