from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.label import Label


T = TypeVar("T", bound="PersonalDetailsLite")


@_attrs_define
class PersonalDetailsLite:
    """Version simplifiée des informations personnelles d'un agent

    Attributes:
        matricule (Union[Unset, str]): Numéro de dossier SIHAM
        civility (Union[Unset, Label]): Etiquette
        surname (Union[Unset, str]): Nom patronymique
        usualname (Union[Unset, str]): Nom usuel
        usual_firstname (Union[Unset, str]): Prénom usuel
        displayname (Union[Unset, str]): Nom d'affichage
    """

    matricule: Union[Unset, str] = UNSET
    civility: Union[Unset, "Label"] = UNSET
    surname: Union[Unset, str] = UNSET
    usualname: Union[Unset, str] = UNSET
    usual_firstname: Union[Unset, str] = UNSET
    displayname: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        matricule = self.matricule
        civility: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.civility, Unset):
            civility = self.civility.to_dict()

        surname = self.surname
        usualname = self.usualname
        usual_firstname = self.usual_firstname
        displayname = self.displayname

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if matricule is not UNSET:
            field_dict["matricule"] = matricule
        if civility is not UNSET:
            field_dict["civility"] = civility
        if surname is not UNSET:
            field_dict["surname"] = surname
        if usualname is not UNSET:
            field_dict["usualname"] = usualname
        if usual_firstname is not UNSET:
            field_dict["usualFirstname"] = usual_firstname
        if displayname is not UNSET:
            field_dict["displayname"] = displayname

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.label import Label

        if not src_dict:
            return None
        d = src_dict.copy()
        matricule = d.pop("matricule", UNSET)

        _civility = d.pop("civility", UNSET)
        civility: Union[Unset, Label]
        if isinstance(_civility, Unset):
            civility = UNSET
        else:
            civility = Label.from_dict(_civility)

        surname = d.pop("surname", UNSET)

        usualname = d.pop("usualname", UNSET)

        usual_firstname = d.pop("usualFirstname", UNSET)

        displayname = d.pop("displayname", UNSET)

        personal_details_lite = cls(
            matricule=matricule,
            civility=civility,
            surname=surname,
            usualname=usualname,
            usual_firstname=usual_firstname,
            displayname=displayname,
        )

        personal_details_lite.additional_properties = d
        return personal_details_lite

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
