import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="Relative")


@_attrs_define
class Relative:
    """Informations minimales sur un enfant d'agent

    Attributes:
        matricule (Union[Unset, str]): Code matricule
        birthday (Union[Unset, datetime.datetime]): Date de naissance
        is_minor (Union[Unset, bool]): Témoin indiquant qu'il s'agit d'un mineur
    """

    matricule: Union[Unset, str] = UNSET
    birthday: Union[Unset, datetime.datetime] = UNSET
    is_minor: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        matricule = self.matricule
        birthday: Union[Unset, str] = UNSET
        if not isinstance(self.birthday, Unset):
            birthday = self.birthday.isoformat()

        is_minor = self.is_minor

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if matricule is not UNSET:
            field_dict["matricule"] = matricule
        if birthday is not UNSET:
            field_dict["birthday"] = birthday
        if is_minor is not UNSET:
            field_dict["isMinor"] = is_minor

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        if not src_dict:
            return None
        d = src_dict.copy()
        matricule = d.pop("matricule", UNSET)

        _birthday = d.pop("birthday", UNSET)
        birthday: Union[Unset, datetime.datetime]
        if isinstance(_birthday, Unset):
            birthday = UNSET
        else:
            birthday = isoparse(_birthday)

        is_minor = d.pop("isMinor", UNSET)

        relative = cls(
            matricule=matricule,
            birthday=birthday,
            is_minor=is_minor,
        )

        relative.additional_properties = d
        return relative

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
