from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.security_input_set_extras import SecurityInputSetExtras


T = TypeVar("T", bound="SecurityInputSet")


@_attrs_define
class SecurityInputSet:
    """Paramètres pour créer des informations de connexion

    Attributes:
        username (Union[Unset, str]): Identifiant
        ip (Union[Unset, str]): Adresse IP d'où devra provenir les requêtes
        subject (Union[Unset, str]): Description du compte
        groups (Union[Unset, List[str]]): Liste de groupes d'accès autorisés (se référer au chemin des ressources)
        extras (Union[Unset, SecurityInputSetExtras]): Informations additionnelles (optionnelles)
    """

    username: Union[Unset, str] = UNSET
    ip: Union[Unset, str] = UNSET
    subject: Union[Unset, str] = UNSET
    groups: Union[Unset, List[str]] = UNSET
    extras: Union[Unset, "SecurityInputSetExtras"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        username = self.username
        ip = self.ip
        subject = self.subject
        groups: Union[Unset, List[str]] = UNSET
        if not isinstance(self.groups, Unset):
            groups = self.groups

        extras: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.extras, Unset):
            extras = self.extras.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if username is not UNSET:
            field_dict["username"] = username
        if ip is not UNSET:
            field_dict["ip"] = ip
        if subject is not UNSET:
            field_dict["subject"] = subject
        if groups is not UNSET:
            field_dict["groups"] = groups
        if extras is not UNSET:
            field_dict["extras"] = extras

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.security_input_set_extras import SecurityInputSetExtras

        if not src_dict:
            return None
        d = src_dict.copy()
        username = d.pop("username", UNSET)

        ip = d.pop("ip", UNSET)

        subject = d.pop("subject", UNSET)

        groups = cast(List[str], d.pop("groups", UNSET))

        _extras = d.pop("extras", UNSET)
        extras: Union[Unset, SecurityInputSetExtras]
        if isinstance(_extras, Unset):
            extras = UNSET
        else:
            extras = SecurityInputSetExtras.from_dict(_extras)

        security_input_set = cls(
            username=username,
            ip=ip,
            subject=subject,
            groups=groups,
            extras=extras,
        )

        security_input_set.additional_properties = d
        return security_input_set

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
