import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="SecurityOutpuSecret")


@_attrs_define
class SecurityOutpuSecret:
    """Données validant la création des informations de connexion

    Attributes:
        date (Union[Unset, datetime.datetime]): Date de délivrance
        issuer (Union[Unset, str]): Origine du reçu
        reference (Union[Unset, str]): Identifiant de référence
        secret (Union[Unset, str]): Mot de passe
    """

    date: Union[Unset, datetime.datetime] = UNSET
    issuer: Union[Unset, str] = UNSET
    reference: Union[Unset, str] = UNSET
    secret: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        date: Union[Unset, str] = UNSET
        if not isinstance(self.date, Unset):
            date = self.date.isoformat()

        issuer = self.issuer
        reference = self.reference
        secret = self.secret

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if date is not UNSET:
            field_dict["date"] = date
        if issuer is not UNSET:
            field_dict["issuer"] = issuer
        if reference is not UNSET:
            field_dict["reference"] = reference
        if secret is not UNSET:
            field_dict["secret"] = secret

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        if not src_dict:
            return None
        d = src_dict.copy()
        _date = d.pop("date", UNSET)
        date: Union[Unset, datetime.datetime]
        if isinstance(_date, Unset):
            date = UNSET
        else:
            date = isoparse(_date)

        issuer = d.pop("issuer", UNSET)

        reference = d.pop("reference", UNSET)

        secret = d.pop("secret", UNSET)

        security_outpu_secret = cls(
            date=date,
            issuer=issuer,
            reference=reference,
            secret=secret,
        )

        security_outpu_secret.additional_properties = d
        return security_outpu_secret

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
