import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.label import Label


T = TypeVar("T", bound="ServiceMode")


@_attrs_define
class ServiceMode:
    """Informations minimales sur la modalité de service

    Attributes:
        matricule (Union[Unset, str]): Numéro de dossier SIHAM
        start (Union[Unset, datetime.datetime]): Date de début
        end (Union[Unset, datetime.datetime]): Date de fin
        planned_end (Union[Unset, datetime.datetime]): Date de fin prévue
        decree (Union[Unset, str]): Arrêté
        mode (Union[Unset, Label]): Etiquette
        fulltime (Union[Unset, bool]): Détermine s'il s'agit d'un temps complet
        validity (Union[Unset, int]): Témoin de validité des informaitons (4 = valide)
        is_pending (Union[Unset, bool]): Détermine si cette modalité n'est pas encore active
        is_expired (Union[Unset, bool]): Détermine si cette modalité est expirée
        is_active (Union[Unset, bool]): Détermine si cette modalité est active
    """

    matricule: Union[Unset, str] = UNSET
    start: Union[Unset, datetime.datetime] = UNSET
    end: Union[Unset, datetime.datetime] = UNSET
    planned_end: Union[Unset, datetime.datetime] = UNSET
    decree: Union[Unset, str] = UNSET
    mode: Union[Unset, "Label"] = UNSET
    fulltime: Union[Unset, bool] = UNSET
    validity: Union[Unset, int] = UNSET
    is_pending: Union[Unset, bool] = UNSET
    is_expired: Union[Unset, bool] = UNSET
    is_active: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        matricule = self.matricule
        start: Union[Unset, str] = UNSET
        if not isinstance(self.start, Unset):
            start = self.start.isoformat()

        end: Union[Unset, str] = UNSET
        if not isinstance(self.end, Unset):
            end = self.end.isoformat()

        planned_end: Union[Unset, str] = UNSET
        if not isinstance(self.planned_end, Unset):
            planned_end = self.planned_end.isoformat()

        decree = self.decree
        mode: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.mode, Unset):
            mode = self.mode.to_dict()

        fulltime = self.fulltime
        validity = self.validity
        is_pending = self.is_pending
        is_expired = self.is_expired
        is_active = self.is_active

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if matricule is not UNSET:
            field_dict["matricule"] = matricule
        if start is not UNSET:
            field_dict["start"] = start
        if end is not UNSET:
            field_dict["end"] = end
        if planned_end is not UNSET:
            field_dict["plannedEnd"] = planned_end
        if decree is not UNSET:
            field_dict["decree"] = decree
        if mode is not UNSET:
            field_dict["mode"] = mode
        if fulltime is not UNSET:
            field_dict["fulltime"] = fulltime
        if validity is not UNSET:
            field_dict["validity"] = validity
        if is_pending is not UNSET:
            field_dict["isPending"] = is_pending
        if is_expired is not UNSET:
            field_dict["isExpired"] = is_expired
        if is_active is not UNSET:
            field_dict["isActive"] = is_active

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.label import Label

        if not src_dict:
            return None
        d = src_dict.copy()
        matricule = d.pop("matricule", UNSET)

        _start = d.pop("start", UNSET)
        start: Union[Unset, datetime.datetime]
        if isinstance(_start, Unset):
            start = UNSET
        else:
            start = isoparse(_start)

        _end = d.pop("end", UNSET)
        end: Union[Unset, datetime.datetime]
        if isinstance(_end, Unset):
            end = UNSET
        else:
            end = isoparse(_end)

        _planned_end = d.pop("plannedEnd", UNSET)
        planned_end: Union[Unset, datetime.datetime]
        if isinstance(_planned_end, Unset):
            planned_end = UNSET
        else:
            planned_end = isoparse(_planned_end)

        decree = d.pop("decree", UNSET)

        _mode = d.pop("mode", UNSET)
        mode: Union[Unset, Label]
        if isinstance(_mode, Unset):
            mode = UNSET
        else:
            mode = Label.from_dict(_mode)

        fulltime = d.pop("fulltime", UNSET)

        validity = d.pop("validity", UNSET)

        is_pending = d.pop("isPending", UNSET)

        is_expired = d.pop("isExpired", UNSET)

        is_active = d.pop("isActive", UNSET)

        service_mode = cls(
            matricule=matricule,
            start=start,
            end=end,
            planned_end=planned_end,
            decree=decree,
            mode=mode,
            fulltime=fulltime,
            validity=validity,
            is_pending=is_pending,
            is_expired=is_expired,
            is_active=is_active,
        )

        service_mode.additional_properties = d
        return service_mode

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
