import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.label import Label


T = TypeVar("T", bound="SituationPopulation")


@_attrs_define
class SituationPopulation:
    """Appartenance d'un agent à une population

    Attributes:
        matricule (Union[Unset, str]): Numéro de dossier
        start_effective (Union[Unset, datetime.datetime]): Date de début
        end_effective (Union[Unset, datetime.datetime]): Date de fin
        population (Union[Unset, Label]): Etiquette
        is_teacher (Union[Unset, bool]): Témoin d'activité d'enseignement
        is_researcher (Union[Unset, bool]): Témoin d'activité de recherche
        is_teacher_or_reasearcher (Union[Unset, bool]): Témoin d'activité de recherche ou d'enseignement
        is_staff (Union[Unset, bool]): Témoin d'activité technique ou administrative (BIATSS)
        is_entilted (Union[Unset, bool]): Témoin du statut de titulaire
        is_intern (Union[Unset, bool]): Témoin du statut de stagiaire
        is_pupil (Union[Unset, bool]): Témoin du statut d'élève
        is_contract (Union[Unset, bool]): Témoin du statut de contractuel
        is_external (Union[Unset, bool]): Témoin du statut d'intervenant extérieur
        is_hosted (Union[Unset, bool]): Témoin du statut d'hébergé
        is_active (Union[Unset, bool]): Témoin d'activité
        is_pending (Union[Unset, bool]): Témoin de mise en attente (pas encore commencé)
        is_expired (Union[Unset, bool]): Témoin d'expiration
        categories (Union[Unset, List[str]]): Catégorisation de la carrière
    """

    matricule: Union[Unset, str] = UNSET
    start_effective: Union[Unset, datetime.datetime] = UNSET
    end_effective: Union[Unset, datetime.datetime] = UNSET
    population: Union[Unset, "Label"] = UNSET
    is_teacher: Union[Unset, bool] = UNSET
    is_researcher: Union[Unset, bool] = UNSET
    is_teacher_or_reasearcher: Union[Unset, bool] = UNSET
    is_staff: Union[Unset, bool] = UNSET
    is_entilted: Union[Unset, bool] = UNSET
    is_intern: Union[Unset, bool] = UNSET
    is_pupil: Union[Unset, bool] = UNSET
    is_contract: Union[Unset, bool] = UNSET
    is_external: Union[Unset, bool] = UNSET
    is_hosted: Union[Unset, bool] = UNSET
    is_active: Union[Unset, bool] = UNSET
    is_pending: Union[Unset, bool] = UNSET
    is_expired: Union[Unset, bool] = UNSET
    categories: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        matricule = self.matricule
        start_effective: Union[Unset, str] = UNSET
        if not isinstance(self.start_effective, Unset):
            start_effective = self.start_effective.isoformat()

        end_effective: Union[Unset, str] = UNSET
        if not isinstance(self.end_effective, Unset):
            end_effective = self.end_effective.isoformat()

        population: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.population, Unset):
            population = self.population.to_dict()

        is_teacher = self.is_teacher
        is_researcher = self.is_researcher
        is_teacher_or_reasearcher = self.is_teacher_or_reasearcher
        is_staff = self.is_staff
        is_entilted = self.is_entilted
        is_intern = self.is_intern
        is_pupil = self.is_pupil
        is_contract = self.is_contract
        is_external = self.is_external
        is_hosted = self.is_hosted
        is_active = self.is_active
        is_pending = self.is_pending
        is_expired = self.is_expired
        categories: Union[Unset, List[str]] = UNSET
        if not isinstance(self.categories, Unset):
            categories = self.categories

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if matricule is not UNSET:
            field_dict["matricule"] = matricule
        if start_effective is not UNSET:
            field_dict["startEffective"] = start_effective
        if end_effective is not UNSET:
            field_dict["endEffective"] = end_effective
        if population is not UNSET:
            field_dict["population"] = population
        if is_teacher is not UNSET:
            field_dict["isTeacher"] = is_teacher
        if is_researcher is not UNSET:
            field_dict["isResearcher"] = is_researcher
        if is_teacher_or_reasearcher is not UNSET:
            field_dict["isTeacherOrReasearcher"] = is_teacher_or_reasearcher
        if is_staff is not UNSET:
            field_dict["isStaff"] = is_staff
        if is_entilted is not UNSET:
            field_dict["isEntilted"] = is_entilted
        if is_intern is not UNSET:
            field_dict["isIntern"] = is_intern
        if is_pupil is not UNSET:
            field_dict["isPupil"] = is_pupil
        if is_contract is not UNSET:
            field_dict["isContract"] = is_contract
        if is_external is not UNSET:
            field_dict["isExternal"] = is_external
        if is_hosted is not UNSET:
            field_dict["isHosted"] = is_hosted
        if is_active is not UNSET:
            field_dict["isActive"] = is_active
        if is_pending is not UNSET:
            field_dict["isPending"] = is_pending
        if is_expired is not UNSET:
            field_dict["isExpired"] = is_expired
        if categories is not UNSET:
            field_dict["categories"] = categories

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.label import Label

        if not src_dict:
            return None
        d = src_dict.copy()
        matricule = d.pop("matricule", UNSET)

        _start_effective = d.pop("startEffective", UNSET)
        start_effective: Union[Unset, datetime.datetime]
        if isinstance(_start_effective, Unset):
            start_effective = UNSET
        else:
            start_effective = isoparse(_start_effective)

        _end_effective = d.pop("endEffective", UNSET)
        end_effective: Union[Unset, datetime.datetime]
        if isinstance(_end_effective, Unset):
            end_effective = UNSET
        else:
            end_effective = isoparse(_end_effective)

        _population = d.pop("population", UNSET)
        population: Union[Unset, Label]
        if isinstance(_population, Unset):
            population = UNSET
        else:
            population = Label.from_dict(_population)

        is_teacher = d.pop("isTeacher", UNSET)

        is_researcher = d.pop("isResearcher", UNSET)

        is_teacher_or_reasearcher = d.pop("isTeacherOrReasearcher", UNSET)

        is_staff = d.pop("isStaff", UNSET)

        is_entilted = d.pop("isEntilted", UNSET)

        is_intern = d.pop("isIntern", UNSET)

        is_pupil = d.pop("isPupil", UNSET)

        is_contract = d.pop("isContract", UNSET)

        is_external = d.pop("isExternal", UNSET)

        is_hosted = d.pop("isHosted", UNSET)

        is_active = d.pop("isActive", UNSET)

        is_pending = d.pop("isPending", UNSET)

        is_expired = d.pop("isExpired", UNSET)

        categories = cast(List[str], d.pop("categories", UNSET))

        situation_population = cls(
            matricule=matricule,
            start_effective=start_effective,
            end_effective=end_effective,
            population=population,
            is_teacher=is_teacher,
            is_researcher=is_researcher,
            is_teacher_or_reasearcher=is_teacher_or_reasearcher,
            is_staff=is_staff,
            is_entilted=is_entilted,
            is_intern=is_intern,
            is_pupil=is_pupil,
            is_contract=is_contract,
            is_external=is_external,
            is_hosted=is_hosted,
            is_active=is_active,
            is_pending=is_pending,
            is_expired=is_expired,
            categories=categories,
        )

        situation_population.additional_properties = d
        return situation_population

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
