import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.discipline import Discipline
    from ..models.job_type import JobType
    from ..models.label import Label


T = TypeVar("T", bound="Speciality")


@_attrs_define
class Speciality:
    """Spécialité d'un agent

    Attributes:
        matricule (Union[Unset, str]): Numéro de dossier
        start_effective (Union[Unset, datetime.datetime]): Date de début
        end_effective (Union[Unset, datetime.datetime]): Date de fin
        speciality (Union[Unset, Label]): Etiquette
        lvl_1_speciality (Union[Unset, Label]): Etiquette
        discipline (Union[Unset, Discipline]): Discipline de spécialité de niveau 1
        profesional_family (Union[Unset, Label]): Etiquette
        job_type (Union[Unset, JobType]): Emploi type
        is_active (Union[Unset, bool]): Témoin d'activité
        is_pending (Union[Unset, bool]): Témoin de mise en attente (pas encore commencé)
        is_expired (Union[Unset, bool]): Témoin d'expiration
    """

    matricule: Union[Unset, str] = UNSET
    start_effective: Union[Unset, datetime.datetime] = UNSET
    end_effective: Union[Unset, datetime.datetime] = UNSET
    speciality: Union[Unset, "Label"] = UNSET
    lvl_1_speciality: Union[Unset, "Label"] = UNSET
    discipline: Union[Unset, "Discipline"] = UNSET
    profesional_family: Union[Unset, "Label"] = UNSET
    job_type: Union[Unset, "JobType"] = UNSET
    is_active: Union[Unset, bool] = UNSET
    is_pending: Union[Unset, bool] = UNSET
    is_expired: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        matricule = self.matricule
        start_effective: Union[Unset, str] = UNSET
        if not isinstance(self.start_effective, Unset):
            start_effective = self.start_effective.isoformat()

        end_effective: Union[Unset, str] = UNSET
        if not isinstance(self.end_effective, Unset):
            end_effective = self.end_effective.isoformat()

        speciality: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.speciality, Unset):
            speciality = self.speciality.to_dict()

        lvl_1_speciality: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.lvl_1_speciality, Unset):
            lvl_1_speciality = self.lvl_1_speciality.to_dict()

        discipline: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.discipline, Unset):
            discipline = self.discipline.to_dict()

        profesional_family: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profesional_family, Unset):
            profesional_family = self.profesional_family.to_dict()

        job_type: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.job_type, Unset):
            job_type = self.job_type.to_dict()

        is_active = self.is_active
        is_pending = self.is_pending
        is_expired = self.is_expired

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if matricule is not UNSET:
            field_dict["matricule"] = matricule
        if start_effective is not UNSET:
            field_dict["startEffective"] = start_effective
        if end_effective is not UNSET:
            field_dict["endEffective"] = end_effective
        if speciality is not UNSET:
            field_dict["speciality"] = speciality
        if lvl_1_speciality is not UNSET:
            field_dict["lvl1Speciality"] = lvl_1_speciality
        if discipline is not UNSET:
            field_dict["discipline"] = discipline
        if profesional_family is not UNSET:
            field_dict["profesionalFamily"] = profesional_family
        if job_type is not UNSET:
            field_dict["jobType"] = job_type
        if is_active is not UNSET:
            field_dict["isActive"] = is_active
        if is_pending is not UNSET:
            field_dict["isPending"] = is_pending
        if is_expired is not UNSET:
            field_dict["isExpired"] = is_expired

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.discipline import Discipline
        from ..models.job_type import JobType
        from ..models.label import Label

        if not src_dict:
            return None
        d = src_dict.copy()
        matricule = d.pop("matricule", UNSET)

        _start_effective = d.pop("startEffective", UNSET)
        start_effective: Union[Unset, datetime.datetime]
        if isinstance(_start_effective, Unset):
            start_effective = UNSET
        else:
            start_effective = isoparse(_start_effective)

        _end_effective = d.pop("endEffective", UNSET)
        end_effective: Union[Unset, datetime.datetime]
        if isinstance(_end_effective, Unset):
            end_effective = UNSET
        else:
            end_effective = isoparse(_end_effective)

        _speciality = d.pop("speciality", UNSET)
        speciality: Union[Unset, Label]
        if isinstance(_speciality, Unset):
            speciality = UNSET
        else:
            speciality = Label.from_dict(_speciality)

        _lvl_1_speciality = d.pop("lvl1Speciality", UNSET)
        lvl_1_speciality: Union[Unset, Label]
        if isinstance(_lvl_1_speciality, Unset):
            lvl_1_speciality = UNSET
        else:
            lvl_1_speciality = Label.from_dict(_lvl_1_speciality)

        _discipline = d.pop("discipline", UNSET)
        discipline: Union[Unset, Discipline]
        if isinstance(_discipline, Unset):
            discipline = UNSET
        else:
            discipline = Discipline.from_dict(_discipline)

        _profesional_family = d.pop("profesionalFamily", UNSET)
        profesional_family: Union[Unset, Label]
        if isinstance(_profesional_family, Unset):
            profesional_family = UNSET
        else:
            profesional_family = Label.from_dict(_profesional_family)

        _job_type = d.pop("jobType", UNSET)
        job_type: Union[Unset, JobType]
        if isinstance(_job_type, Unset):
            job_type = UNSET
        else:
            job_type = JobType.from_dict(_job_type)

        is_active = d.pop("isActive", UNSET)

        is_pending = d.pop("isPending", UNSET)

        is_expired = d.pop("isExpired", UNSET)

        speciality = cls(
            matricule=matricule,
            start_effective=start_effective,
            end_effective=end_effective,
            speciality=speciality,
            lvl_1_speciality=lvl_1_speciality,
            discipline=discipline,
            profesional_family=profesional_family,
            job_type=job_type,
            is_active=is_active,
            is_pending=is_pending,
            is_expired=is_expired,
        )

        speciality.additional_properties = d
        return speciality

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
