import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="Support")


@_attrs_define
class Support:
    """Informations de prise en charge

    Attributes:
        matricule (Union[Unset, str]): Numéro du dossier
        date (Union[Unset, datetime.datetime]): Date de la dernière prise en charge
        count (Union[Unset, int]): Nombre total de prises en charge depuis la mise en production de SIHAM
        is_renewal (Union[Unset, bool]): Témoin de renouvellement (plus d'une prise en charge ou ancienne forme de
            matricule)
    """

    matricule: Union[Unset, str] = UNSET
    date: Union[Unset, datetime.datetime] = UNSET
    count: Union[Unset, int] = UNSET
    is_renewal: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        matricule = self.matricule
        date: Union[Unset, str] = UNSET
        if not isinstance(self.date, Unset):
            date = self.date.isoformat()

        count = self.count
        is_renewal = self.is_renewal

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if matricule is not UNSET:
            field_dict["matricule"] = matricule
        if date is not UNSET:
            field_dict["date"] = date
        if count is not UNSET:
            field_dict["count"] = count
        if is_renewal is not UNSET:
            field_dict["isRenewal"] = is_renewal

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        if not src_dict:
            return None
        d = src_dict.copy()
        matricule = d.pop("matricule", UNSET)

        _date = d.pop("date", UNSET)
        date: Union[Unset, datetime.datetime]
        if isinstance(_date, Unset):
            date = UNSET
        else:
            date = isoparse(_date)

        count = d.pop("count", UNSET)

        is_renewal = d.pop("isRenewal", UNSET)

        support = cls(
            matricule=matricule,
            date=date,
            count=count,
            is_renewal=is_renewal,
        )

        support.additional_properties = d
        return support

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
