from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.client_infos_headers import ClientInfosHeaders


T = TypeVar("T", bound="ClientInfos")


@_attrs_define
class ClientInfos:
    """Description basique d'un client connecté

    Attributes:
        ip (Union[Unset, str]): Adresse IP remontée
        forwarded (Union[Unset, str]): Adresse IP distante
        user_agent (Union[Unset, str]): User agent
        secured_scheme (Union[Unset, bool]): Témoin de connexion par HTTPS
        headers (Union[Unset, ClientInfosHeaders]): En-têtes HTTP reçus du client
        browser (Union[Unset, str]): Navigateur
        os (Union[Unset, str]): Système d'exploitation
        login (Union[Unset, str]): Identifiant de l'utilisateur connecté
        admin (Union[Unset, bool]): Indique si l'utilisateur connecté est administrateur
    """

    ip: Union[Unset, str] = UNSET
    forwarded: Union[Unset, str] = UNSET
    user_agent: Union[Unset, str] = UNSET
    secured_scheme: Union[Unset, bool] = UNSET
    headers: Union[Unset, "ClientInfosHeaders"] = UNSET
    browser: Union[Unset, str] = UNSET
    os: Union[Unset, str] = UNSET
    login: Union[Unset, str] = UNSET
    admin: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        ip = self.ip
        forwarded = self.forwarded
        user_agent = self.user_agent
        secured_scheme = self.secured_scheme
        headers: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.headers, Unset):
            headers = self.headers.to_dict()

        browser = self.browser
        os = self.os
        login = self.login
        admin = self.admin

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if ip is not UNSET:
            field_dict["ip"] = ip
        if forwarded is not UNSET:
            field_dict["forwarded"] = forwarded
        if user_agent is not UNSET:
            field_dict["userAgent"] = user_agent
        if secured_scheme is not UNSET:
            field_dict["securedScheme"] = secured_scheme
        if headers is not UNSET:
            field_dict["headers"] = headers
        if browser is not UNSET:
            field_dict["browser"] = browser
        if os is not UNSET:
            field_dict["os"] = os
        if login is not UNSET:
            field_dict["login"] = login
        if admin is not UNSET:
            field_dict["admin"] = admin

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.client_infos_headers import ClientInfosHeaders

        if not src_dict:
            return None
        d = src_dict.copy()
        ip = d.pop("ip", UNSET)

        forwarded = d.pop("forwarded", UNSET)

        user_agent = d.pop("userAgent", UNSET)

        secured_scheme = d.pop("securedScheme", UNSET)

        _headers = d.pop("headers", UNSET)
        headers: Union[Unset, ClientInfosHeaders]
        if isinstance(_headers, Unset):
            headers = UNSET
        else:
            headers = ClientInfosHeaders.from_dict(_headers)

        browser = d.pop("browser", UNSET)

        os = d.pop("os", UNSET)

        login = d.pop("login", UNSET)

        admin = d.pop("admin", UNSET)

        client_infos = cls(
            ip=ip,
            forwarded=forwarded,
            user_agent=user_agent,
            secured_scheme=secured_scheme,
            headers=headers,
            browser=browser,
            os=os,
            login=login,
            admin=admin,
        )

        client_infos.additional_properties = d
        return client_infos

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
