import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="ServerInfos")


@_attrs_define
class ServerInfos:
    """Informations basiques du serveur

    Attributes:
        os (Union[Unset, str]): Système d'exploitation
        version (Union[Unset, str]): Version du système d'exploitation
        architecture (Union[Unset, str]): Architecture du système d'exploitation
        java (Union[Unset, str]): Version de Java
        server (Union[Unset, str]): Version du serveur
        cores (Union[Unset, int]): Nombre de procésseurs disponibles (bytes)
        free_memory (Union[Unset, int]): Mémoire disponibles (bytes)
        total_memory (Union[Unset, int]): Mémoire utilisable (bytes)
        maximum_memory (Union[Unset, int]): Mémoire disponibles (bytes) (null si illimité)
        start (Union[Unset, datetime.datetime]): Date de démarrage de l'application
        uptime (Union[Unset, int]): Disponibilité du système (millisecondes)
    """

    os: Union[Unset, str] = UNSET
    version: Union[Unset, str] = UNSET
    architecture: Union[Unset, str] = UNSET
    java: Union[Unset, str] = UNSET
    server: Union[Unset, str] = UNSET
    cores: Union[Unset, int] = UNSET
    free_memory: Union[Unset, int] = UNSET
    total_memory: Union[Unset, int] = UNSET
    maximum_memory: Union[Unset, int] = UNSET
    start: Union[Unset, datetime.datetime] = UNSET
    uptime: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        os = self.os
        version = self.version
        architecture = self.architecture
        java = self.java
        server = self.server
        cores = self.cores
        free_memory = self.free_memory
        total_memory = self.total_memory
        maximum_memory = self.maximum_memory
        start: Union[Unset, str] = UNSET
        if not isinstance(self.start, Unset):
            start = self.start.isoformat()

        uptime = self.uptime

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if os is not UNSET:
            field_dict["os"] = os
        if version is not UNSET:
            field_dict["version"] = version
        if architecture is not UNSET:
            field_dict["architecture"] = architecture
        if java is not UNSET:
            field_dict["java"] = java
        if server is not UNSET:
            field_dict["server"] = server
        if cores is not UNSET:
            field_dict["cores"] = cores
        if free_memory is not UNSET:
            field_dict["freeMemory"] = free_memory
        if total_memory is not UNSET:
            field_dict["totalMemory"] = total_memory
        if maximum_memory is not UNSET:
            field_dict["maximumMemory"] = maximum_memory
        if start is not UNSET:
            field_dict["start"] = start
        if uptime is not UNSET:
            field_dict["uptime"] = uptime

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        if not src_dict:
            return None
        d = src_dict.copy()
        os = d.pop("os", UNSET)

        version = d.pop("version", UNSET)

        architecture = d.pop("architecture", UNSET)

        java = d.pop("java", UNSET)

        server = d.pop("server", UNSET)

        cores = d.pop("cores", UNSET)

        free_memory = d.pop("freeMemory", UNSET)

        total_memory = d.pop("totalMemory", UNSET)

        maximum_memory = d.pop("maximumMemory", UNSET)

        _start = d.pop("start", UNSET)
        start: Union[Unset, datetime.datetime]
        if isinstance(_start, Unset):
            start = UNSET
        else:
            start = isoparse(_start)

        uptime = d.pop("uptime", UNSET)

        server_infos = cls(
            os=os,
            version=version,
            architecture=architecture,
            java=java,
            server=server,
            cores=cores,
            free_memory=free_memory,
            total_memory=total_memory,
            maximum_memory=maximum_memory,
            start=start,
            uptime=uptime,
        )

        server_infos.additional_properties = d
        return server_infos

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
